# <markdowncell>
# Create the repressilator from libsbgn.
#
# See on libsbgn-python: https://github.com/matthiaskoenig/libsbgn-python

# <codecell>
from pathlib import Path

import IPython
import libsbgnpy.libsbgn as libsbgn
from IPython.core.display import HTML
from IPython.display import Image
from libsbgnpy.libsbgnTypes import ArcClass, GlyphClass, Language, Orientation
from pygments import highlight
from pygments.formatters import HtmlFormatter
from pygments.lexers import PythonLexer


# <codecell>
def pprint_xml(xml_str: str) -> None:
    """Create highlighted xml."""
    IPython.display.display(
        HTML(
            '<style type="text/css">{}</style>{}'.format(
                HtmlFormatter().get_style_defs(".highlight"),
                highlight(xml_str, PythonLexer(), HtmlFormatter()),
            )
        )
    )


# <codecell>
def create_repressilator(sbgn_path: Path) -> libsbgn.sbgn:
    """Create repressilator SBGN using libsbgn."""
    # create empty sbgn
    sbgn: libsbgn.sbgn = libsbgn.sbgn()

    # create map, set language and set in sbgn
    map = libsbgn.map()
    map.set_language(Language.PD)
    sbgn.set_map(map)

    # create a bounding box for the map
    box = libsbgn.bbox(x=0, y=0, w=363, h=253)
    map.set_bbox(box)

    # create some glyphs
    # glyphs with labels
    g = libsbgn.glyph(class_=GlyphClass.SOURCE_AND_SINK, id="glyph3")
    g.set_bbox(libsbgn.bbox(x=530, y=25, w=36, h=36))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.SOURCE_AND_SINK, id="glyph2")
    g.set_bbox(libsbgn.bbox(x=180, y=25, w=36, h=36))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.SOURCE_AND_SINK, id="glyph11")
    g.set_bbox(libsbgn.bbox(x=455, y=265, w=36, h=36))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.SOURCE_AND_SINK, id="glyph12")
    g.set_bbox(libsbgn.bbox(x=625, y=125, w=36, h=36))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.PROCESS, id="glyph5")
    g.set_bbox(libsbgn.bbox(x=273, y=33, w=20, h=20))
    g.add_port(libsbgn.port(y="43.0", x="263.0", id="glyph5.1"))
    g.add_port(libsbgn.port(y="43.0", x="303.0", id="glyph5.2"))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.SOURCE_AND_SINK, id="glyph10")
    g.set_bbox(libsbgn.bbox(x=545, y=378.61728, w=36, h=36))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.SOURCE_AND_SINK, id="glyph15")
    g.set_bbox(libsbgn.bbox(x=800, y=125, w=36, h=36))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.SOURCE_AND_SINK, id="glyph25")
    g.set_bbox(libsbgn.bbox(x=800, y=450, w=36, h=36))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.PROCESS, id="glyph16")
    g.set_bbox(libsbgn.bbox(x=808, y=228, w=20, h=20))
    g.add_port(libsbgn.port(y="218.0", x="818.0", id="glyph16.1"))
    g.add_port(libsbgn.port(y="258.0", x="818.0", id="glyph16.2"))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.MACROMOLECULE, id="glyph8")
    g.set_label(libsbgn.label(text="Laclp"))
    g.set_bbox(libsbgn.bbox(x=342, y=20.5, w=72, h=45))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.SOURCE_AND_SINK, id="glyph30")
    g.set_bbox(libsbgn.bbox(x=275, y=625, w=36, h=36))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.PROCESS, id="glyph14")
    g.set_bbox(libsbgn.bbox(x=283, y=558, w=20, h=20))
    g.add_port(libsbgn.port(y="588.0", x="293.0", id="glyph14.2"))
    g.add_port(libsbgn.port(y="548.0", x="293.0", id="glyph14.1"))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.SOURCE_AND_SINK, id="glyph17")
    g.set_bbox(libsbgn.bbox(x=115, y=480, w=36, h=36))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.SOURCE_AND_SINK, id="glyph19")
    g.set_bbox(libsbgn.bbox(x=350, y=378.61728, w=36, h=36))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.SOURCE_AND_SINK, id="glyph18")
    g.set_bbox(libsbgn.bbox(x=20, y=378.617, w=36, h=36))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.MACROMOLECULE, id="glyph27")
    g.set_label(libsbgn.label(text="Clp"))
    g.set_bbox(libsbgn.bbox(x=782, y=280.5, w=72, h=45))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.MACROMOLECULE, id="glyph24")
    g.set_label(libsbgn.label(text="Tetp"))
    g.set_bbox(libsbgn.bbox(x=257, y=475.5, w=72, h=45))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.PROCESS, id="glyph13")
    g.set_label(libsbgn.label(text="Laclp"))
    g.set_bbox(libsbgn.bbox(y="386.61728", x="618.0", h="20.0", w="20.0"))
    g.add_port(libsbgn.port(y="396.61728", x="608.0", id="glyph13.1"))
    g.add_port(libsbgn.port(y="396.61728", x="648.0", id="glyph13.2"))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.PROCESS, id="glyph26")
    g.set_label(libsbgn.label(text="Laclp"))
    g.set_bbox(libsbgn.bbox(y="386.61728", x="808.0", h="20.0", w="20.0"))
    g.add_port(libsbgn.port(y="376.61728", x="818.0", id="glyph26.1"))
    g.add_port(libsbgn.port(y="416.61728", x="818.0", id="glyph26.2"))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.SOURCE_AND_SINK, id="glyph22")
    g.set_bbox(libsbgn.bbox(y="528.6667", x="700.0", h="36.0", w="36.0"))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.NUCLEIC_ACID_FEATURE, id="glyph21")
    g.set_label(libsbgn.label(text="Clm"))
    g.set_bbox(libsbgn.bbox(y="374.117", x="682.0", h="45.0", w="72.0"))
    """
    <glyph id="glyph21a" class="unit of information">
                    <label text="ct:mRNA"/>
                    <bbox y="365.367" x="686.25" h="17.5" w="63.5"/>
                </glyph>
    """
    g1 = libsbgn.glyph(id="glyph21a", class_=GlyphClass.UNIT_OF_INFORMATION)

    g1.set_label(libsbgn.label(text="ct:mRNA"))
    g1.set_bbox(libsbgn.bbox(y="365.367", x="686.25", h="17.5", w="63.5"))
    map.add_glyph(g)
    g.add_glyph(g1)
    g = libsbgn.glyph(class_=GlyphClass.PROCESS, id="glyph28")
    g.set_bbox(libsbgn.bbox(y="456.6667", x="708.0", h="20.0", w="20.0"))
    g.add_port(libsbgn.port(y="486.6667", x="718.0", id="glyph28.2"))
    g.add_port(libsbgn.port(y="446.6667", x="718.0", id="glyph28.1"))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.PROCESS, id="glyph32")
    g.set_bbox(libsbgn.bbox(y="386.61728", x="288.0", h="20.0", w="20.0"))
    g.add_port(libsbgn.port(y="396.61728", x="318.0", id="glyph32.2"))
    g.add_port(libsbgn.port(y="396.61728", x="278.0", id="glyph32.1"))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.PROCESS, id="glyph9")
    g.set_bbox(libsbgn.bbox(y="386.61728", x="110.0", h="20.0", w="20.0"))
    g.add_port(libsbgn.port(y="396.61728", x="100.0", id="glyph9.1"))
    g.add_port(libsbgn.port(y="396.61728", x="140.0", id="glyph9.2"))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.NUCLEIC_ACID_FEATURE, id="glyph1")
    g.set_label(libsbgn.label(text="Tetm"))
    g.set_bbox(libsbgn.bbox(y="374.117", x="167.0", h="45.0", w="72.0"))
    g1 = libsbgn.glyph(id="glyph1a", class_=GlyphClass.UNIT_OF_INFORMATION)

    g1.set_label(libsbgn.label(text="ct:mRNA"))
    g1.set_bbox(libsbgn.bbox(y="365.367", x="171.25", h="17.5", w="63.5"))
    """
    <glyph id="glyph1a" class="unit of information">
                    <label text="ct:mRNA"/>
                    <bbox y="365.367" x="171.25" h="17.5" w="63.5"/>
                </glyph>
    """
    map.add_glyph(g)
    g.add_glyph(g1)

    g = libsbgn.glyph(class_=GlyphClass.PROCESS, id="glyph23")
    g.set_bbox(libsbgn.bbox(y="488.0", x="194.0", h="20.0", w="20.0"))
    g.add_port(libsbgn.port(y="498.0", x="184.0", id="glyph23.1"))
    g.add_port(libsbgn.port(y="498.0", x="224.0", id="glyph23.2"))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.PROCESS, id="glyph4")
    g.set_bbox(libsbgn.bbox(y="33.0", x="463.0", h="20.0", w="20.0"))
    g.add_port(libsbgn.port(y="43.0", x="493.0", id="glyph4.2"))
    g.add_port(libsbgn.port(y="43.0", x="453.0", id="glyph4.1"))
    map.add_glyph(g)

    g = libsbgn.glyph(class_=GlyphClass.PROCESS, id="glyph7")
    g.set_label(libsbgn.label(text="Tetm"))
    g.set_bbox(libsbgn.bbox(y="208.0", x="463.0", h="20.0", w="20.0"))
    g.add_port(libsbgn.port(y="238.0", x="473.0", id="glyph7.2"))
    g.add_port(libsbgn.port(y="198.0", x="473.0", id="glyph7.1"))
    map.add_glyph(g)

    # """
    # <glyph id="glyph20" class="nucleic acid feature">
    #     <label text="Laclm"/>
    #     <bbox y="120.5" x="437.0" h="45.0" w="72.0"/>
    #     <glyph id="glyph20a" class="unit of information">
    #         <label text="ct:mRNA"/>
    #         <bbox y="111.75" x="441.25" h="17.5" w="63.5"/>
    #     </glyph>
    # </glyph>
    # """

    g = libsbgn.glyph(class_=GlyphClass.NUCLEIC_ACID_FEATURE, id="glyph20")
    g.set_label(libsbgn.label(text="Laclm"))
    g.set_bbox(libsbgn.bbox(y="120.5", x="437.0", h="45.0", w="72.0"))
    g1 = libsbgn.glyph(id="glyph20a", class_=GlyphClass.UNIT_OF_INFORMATION)

    g1.set_label(libsbgn.label(text="ct:mRNA"))
    g1.set_bbox(libsbgn.bbox(y="111.75", x="441.25", h="17.5", w="63.5"))
    # TODO: add nested glyphs
    # g.add_glyph()
    map.add_glyph(g)
    g.add_glyph(g1)

    g = libsbgn.glyph(class_=GlyphClass.PROCESS, id="glyph6")

    g.set_bbox(libsbgn.bbox(y="133.0", x="553.0", h="20.0", w="20.0"))
    g.add_port(libsbgn.port(y="143.0", x="583.0", id="glyph6.2"))
    g.add_port(libsbgn.port(y="143.0", x="543.0", id="glyph6.1"))
    map.add_glyph(g)

    # g = libsbgn.glyph(class_=GlyphClass.PROCESS, id="glyph6")
    # g.set_label(libsbgn.label(text="Tetm"))
    # g.set_bbox(libsbgn.bbox(y="133.0", x="553.0", h="20.0", w="20.0"))
    # map.add_glyph(g)

    a = libsbgn.arc(
        class_=ArcClass.CONSUMPTION, target="glyph4.2", source="glyph3", id="arc1"
    )
    a.set_start(libsbgn.startType(y="43.0", x="530.0"))
    a.set_end(libsbgn.endType(y="43.0", x="493.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.PRODUCTION, target="glyph2", source="glyph5.1", id="arc4"
    )
    a.set_start(libsbgn.startType(y="43.0", x="263.0"))
    a.set_end(libsbgn.endType(y="43.0", x="216.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.CONSUMPTION, target="glyph7.2", source="glyph11", id="arc6"
    )
    a.set_start(libsbgn.startType(y="265.0", x="473.0"))
    a.set_end(libsbgn.endType(y="238.0", x="473.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.PRODUCTION, target="glyph12", source="glyph6.2", id="arc9"
    )
    a.set_start(libsbgn.startType(y="143.0", x="583.0"))
    a.set_end(libsbgn.endType(y="143.0", x="625.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.CONSUMPTION, target="glyph5.2", source="glyph8", id="arc3"
    )
    a.set_start(libsbgn.startType(y="43.0", x="342.0"))
    a.set_end(libsbgn.endType(y="43.0", x="303.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.CONSUMPTION, target="glyph13.1", source="glyph10", id="arc19"
    )
    a.set_start(libsbgn.startType(y="396.61728", x="581.0"))
    a.set_end(libsbgn.endType(y="396.61728", x="608.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.PRODUCTION, target="glyph8", source="glyph4.1", id="arc22"
    )
    a.set_start(libsbgn.startType(y="43.0", x="453.0"))
    a.set_end(libsbgn.endType(y="43.0", x="414.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.CONSUMPTION, target="glyph15", source="glyph16.1", id="arc24"
    )
    a.set_start(libsbgn.startType(y="218.0", x="818.0"))
    a.set_end(libsbgn.endType(y="161.0", x="818.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.PRODUCTION, target="glyph27", source="glyph26.1", id="arc10"
    )
    a.set_start(libsbgn.startType(y="376.61728", x="818.0"))
    a.set_end(libsbgn.endType(y="325.5", x="818.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.CONSUMPTION, target="glyph26.2", source="glyph25", id="arc11"
    )
    a.set_start(libsbgn.startType(y="450.0", x="818.0"))
    a.set_end(libsbgn.endType(y="416.61728", x="818.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.CONSUMPTION, target="glyph16.2", source="glyph27", id="arc13"
    )
    a.set_start(libsbgn.startType(y="280.5", x="818.0"))
    a.set_end(libsbgn.endType(y="258.0", x="818.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.INHIBITION, target="glyph32", source="glyph8", id="arc15"
    )
    a.set_start(libsbgn.startType(y="65.5", x="353.74677"))
    a.set_end(libsbgn.endType(y="386.61728", x="297.89154"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.PRODUCTION, target="glyph30", source="glyph14.2", id="arc16"
    )
    a.set_start(libsbgn.startType(y="588.0", x="293.0"))
    a.set_end(libsbgn.endType(y="625.0", x="293.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.CONSUMPTION, target="glyph14.1", source="glyph24", id="arc17"
    )
    a.set_start(libsbgn.startType(y="520.5", x="293.0"))
    a.set_end(libsbgn.endType(y="548.0", x="293.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.CONSUMPTION, target="glyph23.1", source="glyph17", id="arc20"
    )
    a.set_start(libsbgn.startType(y="498.0", x="151.0"))
    a.set_end(libsbgn.endType(y="498.0", x="184.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.PRODUCTION, target="glyph24", source="glyph23.2", id="arc21"
    )
    a.set_start(libsbgn.startType(y="498.0", x="224.0"))
    a.set_end(libsbgn.endType(y="498.0", x="257.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.CONSUMPTION, target="glyph32.2", source="glyph19", id="arc25"
    )
    a.set_start(libsbgn.startType(y="396.61728", x="350.0"))
    a.set_end(libsbgn.endType(y="396.61728", x="318.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.PRODUCTION, target="glyph18", source="glyph9.1", id="arc28"
    )
    a.set_start(libsbgn.startType(y="396.61728", x="100.0"))
    a.set_end(libsbgn.endType(y="396.617", x="56.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.INHIBITION, target="glyph7", source="glyph27", id="arc2"
    )
    a.set_start(libsbgn.startType(y="280.5", x="791.871"))
    a.set_end(libsbgn.endType(y="217.66245", x="483.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.INHIBITION, target="glyph13", source="glyph24", id="arc23"
    )
    a.set_start(libsbgn.startType(y="498.77063", x="329.0"))
    a.set_end(libsbgn.endType(y="406.61728", x="627.2619"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.PRODUCTION, target="glyph21", source="glyph13.2", id="arc5"
    )
    a.set_start(libsbgn.startType(y="396.61728", x="648.0"))
    a.set_end(libsbgn.endType(y="396.617", x="682.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.NECESSARY_STIMULATION,
        target="glyph26",
        source="glyph21",
        id="arc7",
    )
    a.set_start(libsbgn.startType(y="396.617", x="754.0"))
    a.set_end(libsbgn.endType(y="396.61728", x="808.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.PRODUCTION, target="glyph22", source="glyph28.2", id="arc8"
    )
    a.set_start(libsbgn.startType(y="486.6667", x="718.0"))
    a.set_end(libsbgn.endType(y="528.6667", x="718.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.CONSUMPTION, target="glyph28.1", source="glyph21", id="arc12"
    )
    a.set_start(libsbgn.startType(y="419.117", x="718.0"))
    a.set_end(libsbgn.endType(y="446.6667", x="718.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.PRODUCTION, target="glyph1", source="glyph32.1", id="arc14"
    )
    a.set_start(libsbgn.startType(y="396.61728", x="278.0"))
    a.set_end(libsbgn.endType(y="396.617", x="239.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.CONSUMPTION, target="glyph9.2", source="glyph1", id="arc18"
    )
    a.set_start(libsbgn.startType(y="396.617", x="167.0"))
    a.set_end(libsbgn.endType(y="396.61728", x="140.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.NECESSARY_STIMULATION,
        target="glyph23",
        source="glyph1",
        id="arc26",
    )
    a.set_start(libsbgn.startType(y="419.117", x="203.22192"))
    a.set_end(libsbgn.endType(y="488.0", x="203.90137"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.NECESSARY_STIMULATION,
        target="glyph4",
        source="glyph20",
        id="arc27",
    )
    a.set_start(libsbgn.startType(y="120.5", x="473.0"))
    a.set_end(libsbgn.endType(y="53.0", x="473.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.PRODUCTION, target="glyph20", source="glyph7.1", id="arc29"
    )
    a.set_start(libsbgn.startType(y="198.0", x="473.0"))
    a.set_end(libsbgn.endType(y="165.5", x="473.0"))
    map.add_arc(a)

    a = libsbgn.arc(
        class_=ArcClass.CONSUMPTION, target="glyph6.1", source="glyph20", id="arc30"
    )
    a.set_start(libsbgn.startType(y="143.0", x="509.0"))
    a.set_end(libsbgn.endType(y="143.0", x="543.0"))
    map.add_arc(a)

    f_out = str(sbgn_path)
    sbgn.write_file(f_out)

    # render SBGN
    from libsbgnpy import render

    f_png: str = str(sbgn_path.parent / f"{sbgn_path.stem}.png")
    render.render_sbgn(sbgn, image_file=f_png, file_format="png")
    Image(f_png, width=300)
    return sbgn


# <codecell>
if __name__ == "__main__":
    from combine_notebooks import RESULTS_DIR

    RESULTS_DIR
    sbgn: libsbgn.sbgn = create_repressilator(
        sbgn_path=RESULTS_DIR / "repressilator.sbgn"
    )



"""Runs all the example scripts and generates all COMBINE files."""
from pathlib import Path

from combine_notebooks.examples.example_biopax import create_repressilator as biopax
from combine_notebooks.examples.example_cellml import create_repressilator as cellml
from combine_notebooks.examples.example_omex import create_omex
from combine_notebooks.examples.example_sbgn import create_repressilator as sbgn
from combine_notebooks.examples.example_sbml_antimony import (
    create_repressilator as sbml_antimony,
)
from combine_notebooks.examples.example_sbml_libsbml import (
    create_repressilator as sbml_libsbml,
)
from combine_notebooks.examples.example_sbml_sbmlutils import (
    create_repressilator as sbml_sbmlutils,
)
from combine_notebooks.examples.example_sedml import create_repressilator as sedml


def create_combine_files(output_dir: Path) -> None:
    """Create all the COMBINE files."""
    biopax(biopax_path=output_dir / "repressilator_biopax.owl")
    cellml(cellml_path=output_dir / "repressilator_cellml.cellml")
    sbgn(sbgn_path=output_dir / "repressilator_sbgn.sbgn")
    sedml(sedml_path=output_dir / "repressilator_sedml.xml")
    sbml_antimony(sbml_path=output_dir / "repressilator_sbml_antimony.xml")
    sbml_libsbml(sbml_path=output_dir / "repressilator_sbml_libsbml.xml")
    sbml_sbmlutils(sbml_path=output_dir / "repressilator_sbml_sbmlutils.xml")


if __name__ == "__main__":
    from combine_notebooks import RESULTS_DIR

    create_combine_files(output_dir=RESULTS_DIR)
    create_omex(omex_path=RESULTS_DIR / "combine.omex", results_dir=RESULTS_DIR)



# <markdowncell>
# Create repressilator with antimony.

# http://antimony.sourceforge.net/
# https://tellurium.readthedocs.io/en/latest/antimony.html

# <codecell>
from pathlib import Path

import antimony
import libsbml

from combine_notebooks.validation.validation_sbml import validate_sbml


# <codecell>
def create_repressilator(sbml_path: Path) -> libsbml.SBMLDocument:
    """Create repressilator with antimony."""
    model: str = """
        model antimony_repressilator
            # compartment
            compartment cell = 1
            # species
            species PX = 0
            species PY = 0
            species PZ = 0
            species X = 0
            species Y = 20
            species Z = 0
            # parameters
            beta = 0.2
            alpha0 = 0.2164
            alpha = 216.404
            eff = 20
            n = 2
            KM = 40
            tau_mRNA = 2
            tau_prot = 10
            ps_a = 0.5
            ps_0 = 0.0005
            # assignment rules
            t_ave := tau_mRNA/ln(2);
            beta := tau_mRNA/tau_prot;
            k_tl := eff/t_ave;
            a_tr := (ps_a - ps_0)*60;
            a0_tr := ps_0*60;
            kd_prot := ln(2)/tau_prot;
            kd_mRNA := ln(2)/tau_mRNA;
            alpha := a_tr*eff*tau_prot/(ln(2)*KM);
            alpha0 := (a0_tr*eff*tau_prot)/(ln(2)*KM);

            # reactions
            X ->; kd_mRNA*X;
            Y ->; kd_mRNA*Y;
            Z ->; kd_mRNA*Z;
            -> PX; k_tl*X;
            -> PY; k_tl*Y;
            -> PZ; k_tl*Z;
            PX ->; kd_prot*PX;
            PY ->; kd_prot*PY;
            PZ ->; kd_prot*PZ;
            -> X; a0_tr +a_tr *KM^n/(KM^n +PZ ^n);
            -> Y; a0_tr +a_tr *KM^n/(KM^n +PX ^n);
            -> Z; a0_tr +a_tr *KM^n/(KM^n +PY ^n);

            # annotations
            cell identity "http://identifiers.org/SBO:0000290"
            cell identity "http://identifiers.org/GO:0005623"
            PX identity "http://identifiers.org/SBO:0000252"
            PX identity "http://identifiers.org/uniprot/P03023"
            PY identity "http://identifiers.org/SBO:0000252"
            PY identity "http://identifiers.org/uniprot/P04483"
            PZ identity "http://identifiers.org/SBO:0000252"
            PZ identity "http://identifiers.org/uniprot/P03034"
            X identity "http://identifiers.org/SBO:0000250"
            X identity "http://identifiers.org/CHEBI:33699"
            X identity "http://identifiers.org/kegg.compound/C00046"
            X identity "http://identifiers.org/uniprot/P03023"
            Y identity "http://identifiers.org/SBO:0000250"
            Y identity "http://identifiers.org/CHEBI:33699"
            Y identity "http://identifiers.org/kegg.compound/C00046"
            Y identity "http://identifiers.org/uniprot/P04483"
            Z identity "http://identifiers.org/SBO:0000250"
            Z identity "http://identifiers.org/CHEBI:33699"
            Z identity "http://identifiers.org/kegg.compound/C00046"
            Z identity "http://identifiers.org/uniprot/P03034"
            tau_mRNA identity "http://identifiers.org/SBO:0000356"
            k_tl identity "http://identifiers.org/SBO:0000016"
            t_ave identity "http://identifiers.org/SBO:0000348"
            kd_prot identity "http://identifiers.org/SBO:0000356"
            a0_tr identity "http://identifiers.org/SBO:0000485"
            a_tr identity "http://identifiers.org/SBO:0000186"
            _J0 identity "http://identifiers.org/SBO:0000179"
            _J0 identity "http://identifiers.org/GO:0006402"
            _J1 identity "http://identifiers.org/SBO:0000179"
            _J1 identity "http://identifiers.org/GO:0006402"
            _J2 identity "http://identifiers.org/SBO:0000179"
            _J2 identity "http://identifiers.org/GO:0006402"
            _J3 identity "http://identifiers.org/SBO:0000184"
            _J3 identity "http://identifiers.org/GO:0006412"
            _J4 identity "http://identifiers.org/SBO:0000184"
            _J4 identity "http://identifiers.org/GO:0006412"
            _J5 identity "http://identifiers.org/SBO:0000184"
            _J5 identity "http://identifiers.org/GO:0006412"
            _J6 identity "http://identifiers.org/SBO:0000179"
            _J6 identity "http://identifiers.org/GO:0030163"
            _J7 identity "http://identifiers.org/SBO:0000179"
            _J7 identity "http://identifiers.org/GO:0030163"
            _J8 identity "http://identifiers.org/SBO:0000179"
            _J8 identity "http://identifiers.org/GO:0030163"
            _J9 identity "http://identifiers.org/SBO:0000183"
            _J9 identity "http://identifiers.org/GO:0006351"
            _J10 identity "http://identifiers.org/SBO:0000183"
            _J10 identity "http://identifiers.org/GO:0006351"
            _J11 identity "http://identifiers.org/SBO:0000184"
            _J11 identity "http://identifiers.org/GO:0006351"

        end
    """
    print(model)

    status = antimony.loadAntimonyString(model)
    print(f"Antimony status: {status}")
    print(antimony.getLastError())
    sbml_str = antimony.getSBMLString()
    print(sbml_str)
    print(antimony.getSBMLWarnings())

    # validation
    doc = libsbml.readSBMLFromString(sbml_str)
    validate_sbml(doc)

    # write SBML
    libsbml.writeSBMLToFile(doc, str(sbml_path))

    return doc


# <codecell>
if __name__ == "__main__":
    from combine_notebooks import RESULTS_DIR

    create_repressilator(sbml_path=RESULTS_DIR / "repressilator_antimony.xml")



# <markdowncell>
# Example script for creating SBML repressilator with cellml.
# <markdowncell>
# https://www.cellml.org/getting-started/tutorials/COR-tutorial
# https://models.physiomeproject.org/exposure/bee9fb9df261ba69b51897f2e49b1691/elowitz_leibler_2000.cellml/view?searchterm=repressilator
#
# https://libcellml.org/documentation/v0.2.0/user/tutorials/tutorial3/index
# <markdowncell>
# opencor: https://opencor.ws/downloads/index.html
#
# Issues: https://github.com/cellml/libcellml/issues/1017

# <codecell>
from pathlib import Path

import libcellml
import libsbml

from combine_notebooks.cellml_utilities import print_model


# <codecell>
def create_repressilator(cellml_path: Path) -> libcellml.Model:
    """Create CellML repressilator and save to Path."""
    model = libcellml.Model()
    model.setName("repressilator")

    """
    <units name="minute">
      <unit units="second" multiplier="60"/>
    </units>
    <units name="first_order_rate_constant">
        <unit units="minute" exponent="-1"/>
    </units>
    """
    minute = libcellml.Units("minute")
    minute.addUnit("second", 1, 60)
    model.addUnits(minute)

    # first_order_rate_constant = libcellml.Units("minute")
    # first_order_rate_constant.addUnit("minute")
    # model.addUnits(first_order_rate_constant)
    ##############################################################################################
    """
    <component name="environment">
      <variable units="minute" public_interface="out" name="time" cmeta:id="environment_time"/>
    </component>
    """
    comp_env = libcellml.Component()
    comp_env.setName("environment")
    model.addComponent(comp_env)

    var_n = libcellml.Variable()
    var_n.setName("time")
    var_n.setInterfaceType("out")
    var_n.setUnits("minute")
    var_n.setId("environment_time")
    comp_env.addVariable(var_n)

    comp_parameters = libcellml.Component()
    comp_parameters.setName("parameters")
    model.addComponent(comp_parameters)

    # var n: dimensionless {init: 2, pub: out};
    var_n = libcellml.Variable()
    var_n.setName("n")
    var_n.setInitialValue(2.0)
    var_n.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_n)

    var_alpha_0 = libcellml.Variable()
    var_alpha_0.setName("alpha_0")
    var_alpha_0.setInitialValue(0.216)
    var_alpha_0.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_alpha_0)

    var_alpha = libcellml.Variable()
    var_alpha.setName("alpha")
    var_alpha.setInitialValue(216)
    var_alpha.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_alpha)

    var_beta = libcellml.Variable()
    var_beta.setName("beta")
    var_beta.setInitialValue(0.2)
    var_beta.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_beta)

    var_K_m = libcellml.Variable()
    var_K_m.setName("K_m")
    var_K_m.setInitialValue(40)
    var_K_m.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_K_m)

    var_efficiency = libcellml.Variable()
    var_efficiency.setName("efficiency")
    var_efficiency.setInitialValue(20)
    var_efficiency.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_efficiency)

    var_mRNA_halflife = libcellml.Variable()
    var_mRNA_halflife.setName("mRNA_halflife")
    var_mRNA_halflife.setInitialValue(2)
    var_mRNA_halflife.setUnits("minute")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_mRNA_halflife)

    var_t_ave = libcellml.Variable()
    var_t_ave.setName("t_ave")
    var_t_ave.setUnits("minute")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_t_ave)

    # var_eff = libcellml.Variable()
    # var_eff.setName("eff")
    # var_eff.setInitialValue(20)
    # var_eff.setUnits("minute")
    # comp_parameters.addVariable(var_eff)

    # start with math
    math_ast1: libsbml.ASTNode = libsbml.parseL3Formula("mRNA_halflife/ln(2)")
    math_str = libsbml.writeMathMLToString(math_ast1)
    print(math_str)
    math_header = '<math xmlns="http://www.w3.org/1998/Math/MathML" xmlns:cellml="http://www.cellml.org/cellml/2.0#">'
    #    Include the MathML strings in the component.
    math_str = "\n".join([math_header] + math_str.split("\n")[2:])
    print(math_str)
    comp_parameters.setMath(math_str)

    ##############################################################################################

    comp_parameters = libcellml.Component()
    comp_parameters.setName("M_lacl")
    model.addComponent(comp_parameters)

    # var n: dimensionless {init: 2, pub: out};
    var_n = libcellml.Variable()
    var_n.setName("M_lacl")
    var_n.setInitialValue(5.0)
    var_n.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_n)

    var_alpha = libcellml.Variable()
    var_alpha.setName("alpha")
    var_alpha.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_alpha)

    var_beta = libcellml.Variable()
    var_beta.setName("P_cl")
    # var_beta.setInitialValue(0.2)
    var_beta.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_beta)

    var_beta = libcellml.Variable()
    var_beta.setName("n")
    # var_beta.setInitialValue(0.2)
    var_beta.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_beta)

    var_alpha_0 = libcellml.Variable()
    var_alpha_0.setName("alpha_0")
    var_alpha_0.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_alpha_0)

    var_K_m = libcellml.Variable()
    var_K_m.setName("K_m")
    # var_K_m.setInitialValue(40)
    var_K_m.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_K_m)

    var_efficiency = libcellml.Variable()
    var_efficiency.setName("efficiency")
    # var_efficiency.setInitialValue(20)
    var_efficiency.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_efficiency)

    var_t_ave = libcellml.Variable()
    var_t_ave.setName("t_ave")
    var_t_ave.setUnits("minute")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_t_ave)

    var_mRNA_halflife = libcellml.Variable()
    var_mRNA_halflife.setName("time")
    # var_mRNA_halflife.setInitialValue(2)
    var_mRNA_halflife.setUnits("minute")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_mRNA_halflife)

    # var_eff = libcellml.Variable()
    # var_eff.setName("eff")
    # var_eff.setInitialValue(20)
    # var_eff.setUnits("minute")
    # comp_parameters.addVariable(var_eff)

    # start with math
    math_ast1 = libsbml.parseL3Formula(
        "a0_tr + (a_tr * power(KM, n)) / (power(KM, n) + power(PZ, n))"
    )
    math_str = libsbml.writeMathMLToString(math_ast1)
    print(math_str)
    math_header = '<math xmlns="http://www.w3.org/1998/Math/MathML" xmlns:cellml="http://www.cellml.org/cellml/2.0#">'
    #    Include the MathML strings in the component.
    math_str = "\n".join([math_header] + math_str.split("\n")[2:])
    print(math_str)
    comp_parameters.setMath(math_str)

    ##############################################################################################

    comp_parameters = libcellml.Component()
    comp_parameters.setName("M_tetR")
    model.addComponent(comp_parameters)

    # var n: dimensionless {init: 2, pub: out};
    var_n = libcellml.Variable()
    var_n.setName("M_tetR")
    var_n.setInitialValue(0.0)
    var_n.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_n)

    var_alpha = libcellml.Variable()
    var_alpha.setName("alpha")
    var_alpha.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_alpha)

    var_beta = libcellml.Variable()
    var_beta.setName("P_lacl")
    # var_beta.setInitialValue(0.2)
    var_beta.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_beta)

    var_beta = libcellml.Variable()
    var_beta.setName("n")
    # var_beta.setInitialValue(0.2)
    var_beta.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_beta)

    var_alpha_0 = libcellml.Variable()
    var_alpha_0.setName("alpha_0")
    var_alpha_0.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_alpha_0)

    var_K_m = libcellml.Variable()
    var_K_m.setName("K_m")
    # var_K_m.setInitialValue(40)
    var_K_m.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_K_m)

    var_efficiency = libcellml.Variable()
    var_efficiency.setName("efficiency")
    # var_efficiency.setInitialValue(20)
    var_efficiency.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_efficiency)

    var_t_ave = libcellml.Variable()
    var_t_ave.setName("t_ave")
    var_t_ave.setUnits("minute")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_t_ave)

    var_mRNA_halflife = libcellml.Variable()
    var_mRNA_halflife.setName("time")
    # var_mRNA_halflife.setInitialValue(2)
    var_mRNA_halflife.setUnits("minute")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_mRNA_halflife)

    # var_eff = libcellml.Variable()
    # var_eff.setName("eff")
    # var_eff.setInitialValue(20)
    # var_eff.setUnits("minute")
    # comp_parameters.addVariable(var_eff)

    # start with math
    math_ast1 = libsbml.parseL3Formula(
        "a0_tr + (a_tr * power(KM, n)) / (power(KM, n) + power(PZ, n))"
    )
    math_str = libsbml.writeMathMLToString(math_ast1)
    print(math_str)
    math_header = '<math xmlns="http://www.w3.org/1998/Math/MathML" xmlns:cellml="http://www.cellml.org/cellml/2.0#">'
    #    Include the MathML strings in the component.
    math_str = "\n".join([math_header] + math_str.split("\n")[2:])
    print(math_str)
    comp_parameters.setMath(math_str)

    ##############################################################################################

    comp_parameters = libcellml.Component()
    comp_parameters.setName("M_cl")
    model.addComponent(comp_parameters)

    # var n: dimensionless {init: 2, pub: out};
    var_n = libcellml.Variable()
    var_n.setName("M_cl")
    var_n.setInitialValue(15.0)
    var_n.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_n)

    var_alpha = libcellml.Variable()
    var_alpha.setName("alpha")
    var_alpha.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_alpha)

    var_beta = libcellml.Variable()
    var_beta.setName("P_tetR")
    # var_beta.setInitialValue(0.2)
    var_beta.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_beta)

    var_beta = libcellml.Variable()
    var_beta.setName("n")
    # var_beta.setInitialValue(0.2)
    var_beta.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_beta)

    var_alpha_0 = libcellml.Variable()
    var_alpha_0.setName("alpha_0")
    var_alpha_0.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_alpha_0)

    var_K_m = libcellml.Variable()
    var_K_m.setName("K_m")
    # var_K_m.setInitialValue(40)
    var_K_m.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_K_m)

    var_efficiency = libcellml.Variable()
    var_efficiency.setName("efficiency")
    # var_efficiency.setInitialValue(20)
    var_efficiency.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_efficiency)

    var_t_ave = libcellml.Variable()
    var_t_ave.setName("t_ave")
    var_t_ave.setUnits("minute")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_t_ave)

    var_mRNA_halflife = libcellml.Variable()
    var_mRNA_halflife.setName("time")
    # var_mRNA_halflife.setInitialValue(2)
    var_mRNA_halflife.setUnits("minute")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_mRNA_halflife)

    # var_eff = libcellml.Variable()
    # var_eff.setName("eff")
    # var_eff.setInitialValue(20)
    # var_eff.setUnits("minute")
    # comp_parameters.addVariable(var_eff)

    # start with math
    math_ast1 = libsbml.parseL3Formula(
        "a0_tr + (a_tr * power(KM, n)) / (power(KM, n) + power(PZ, n))"
    )
    math_str = libsbml.writeMathMLToString(math_ast1)
    print(math_str)
    math_header = '<math xmlns="http://www.w3.org/1998/Math/MathML" xmlns:cellml="http://www.cellml.org/cellml/2.0#">'
    #    Include the MathML strings in the component.
    math_str = "\n".join([math_header] + math_str.split("\n")[2:])
    print(math_str)
    comp_parameters.setMath(math_str)

    ##############################################################################################

    comp_parameters = libcellml.Component()
    comp_parameters.setName("P_lacl")
    model.addComponent(comp_parameters)

    # var n: dimensionless {init: 2, pub: out};
    var_n = libcellml.Variable()
    var_n.setName("P_lacl")
    var_n.setInitialValue(0.0)
    var_n.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_n)

    var_alpha = libcellml.Variable()
    var_alpha.setName("beta")
    var_alpha.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_alpha)

    var_beta = libcellml.Variable()
    var_beta.setName("M_lacl")
    # var_beta.setInitialValue(0.2)
    var_beta.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_beta)

    var_t_ave = libcellml.Variable()
    var_t_ave.setName("t_ave")
    var_t_ave.setUnits("minute")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_t_ave)

    var_K_m = libcellml.Variable()
    var_K_m.setName("K_m")
    # var_K_m.setInitialValue(40)
    var_K_m.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_K_m)

    var_efficiency = libcellml.Variable()
    var_efficiency.setName("efficiency")
    # var_efficiency.setInitialValue(20)
    var_efficiency.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_efficiency)

    var_mRNA_halflife = libcellml.Variable()
    var_mRNA_halflife.setName("time")
    # var_mRNA_halflife.setInitialValue(2)
    var_mRNA_halflife.setUnits("minute")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_mRNA_halflife)

    # var_eff = libcellml.Variable()
    # var_eff.setName("eff")
    # var_eff.setInitialValue(20)
    # var_eff.setUnits("minute")
    # comp_parameters.addVariable(var_eff)

    # start with math
    math_ast1 = libsbml.parseL3Formula(
        "a0_tr + (a_tr * power(KM, n)) / (power(KM, n) + power(PZ, n))"
    )
    math_str = libsbml.writeMathMLToString(math_ast1)
    print(math_str)
    math_header = '<math xmlns="http://www.w3.org/1998/Math/MathML" xmlns:cellml="http://www.cellml.org/cellml/2.0#">'
    #    Include the MathML strings in the component.
    math_str = "\n".join([math_header] + math_str.split("\n")[2:])
    print(math_str)
    comp_parameters.setMath(math_str)

    ##############################################################################################

    comp_parameters = libcellml.Component()
    comp_parameters.setName("P_tetR")
    model.addComponent(comp_parameters)

    # var n: dimensionless {init: 2, pub: out};
    var_n = libcellml.Variable()
    var_n.setName("P_tetR")
    var_n.setInitialValue(0.0)
    var_n.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_n)

    var_alpha = libcellml.Variable()
    var_alpha.setName("beta")
    var_alpha.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_alpha)

    var_beta = libcellml.Variable()
    var_beta.setName("M_tetR")
    # var_beta.setInitialValue(0.2)
    var_beta.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_beta)

    var_t_ave = libcellml.Variable()
    var_t_ave.setName("t_ave")
    var_t_ave.setUnits("minute")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_t_ave)

    var_K_m = libcellml.Variable()
    var_K_m.setName("K_m")
    # var_K_m.setInitialValue(40)
    var_K_m.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_K_m)

    var_efficiency = libcellml.Variable()
    var_efficiency.setName("efficiency")
    # var_efficiency.setInitialValue(20)
    var_efficiency.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_efficiency)

    var_mRNA_halflife = libcellml.Variable()
    var_mRNA_halflife.setName("time")
    # var_mRNA_halflife.setInitialValue(2)
    var_mRNA_halflife.setUnits("minute")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_mRNA_halflife)

    # var_eff = libcellml.Variable()
    # var_eff.setName("eff")
    # var_eff.setInitialValue(20)
    # var_eff.setUnits("minute")
    # comp_parameters.addVariable(var_eff)

    # start with math
    math_ast1 = libsbml.parseL3Formula(
        "a0_tr + (a_tr * power(KM, n)) / (power(KM, n) + power(PZ, n))"
    )
    math_str = libsbml.writeMathMLToString(math_ast1)
    print(math_str)
    math_header = '<math xmlns="http://www.w3.org/1998/Math/MathML" xmlns:cellml="http://www.cellml.org/cellml/2.0#">'
    #    Include the MathML strings in the component.
    math_str = "\n".join([math_header] + math_str.split("\n")[2:])
    print(math_str)
    comp_parameters.setMath(math_str)

    ##############################################################################################

    comp_parameters = libcellml.Component()
    comp_parameters.setName("P_cl")
    model.addComponent(comp_parameters)

    # var n: dimensionless {init: 2, pub: out};
    var_n = libcellml.Variable()
    var_n.setName("P_cl")
    var_n.setInitialValue(0.0)
    var_n.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_n)

    var_alpha = libcellml.Variable()
    var_alpha.setName("beta")
    var_alpha.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_alpha)

    var_beta = libcellml.Variable()
    var_beta.setName("M_cl")
    # var_beta.setInitialValue(0.2)
    var_beta.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_beta)

    var_t_ave = libcellml.Variable()
    var_t_ave.setName("t_ave")
    var_t_ave.setUnits("minute")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_t_ave)

    var_K_m = libcellml.Variable()
    var_K_m.setName("K_m")
    # var_K_m.setInitialValue(40)
    var_K_m.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_K_m)

    var_efficiency = libcellml.Variable()
    var_efficiency.setName("efficiency")
    # var_efficiency.setInitialValue(20)
    var_efficiency.setUnits("dimensionless")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_efficiency)

    var_mRNA_halflife = libcellml.Variable()
    var_mRNA_halflife.setName("time")
    # var_mRNA_halflife.setInitialValue(2)
    var_mRNA_halflife.setUnits("minute")
    # var_n.setInterfaceType()
    comp_parameters.addVariable(var_mRNA_halflife)

    # var_eff = libcellml.Variable()
    # var_eff.setName("eff")
    # var_eff.setInitialValue(20)
    # var_eff.setUnits("minute")
    # comp_parameters.addVariable(var_eff)

    # start with math
    math_ast1 = libsbml.parseL3Formula(
        "a0_tr + (a_tr * power(KM, n)) / (power(KM, n) + power(PZ, n))"
    )
    math_str = libsbml.writeMathMLToString(math_ast1)
    print(math_str)
    math_header = '<math xmlns="http://www.w3.org/1998/Math/MathML" xmlns:cellml="http://www.cellml.org/cellml/2.0#">'
    #    Include the MathML strings in the component.
    math_str = "\n".join([math_header] + math_str.split("\n")[2:])
    print(math_str)
    comp_parameters.setMath(math_str)
    ############################################################

    comp_env = libcellml.Component()

    model.addComponent(comp_env)

    #  Checking that it worked
    print("-" * 80)
    print_model(model)
    print("-" * 80)

    # Save model
    printer = libcellml.Printer()
    serialised_model: str = printer.printModel(model)
    print("-" * 80)
    print(serialised_model)
    print("-" * 80)

    with open(cellml_path, "w") as f_cellml:
        f_cellml.write(serialised_model)

    return model


# <codecell>
def validate_model(model: libcellml.Model) -> None:
    """Validate existing model."""
    # Validation
    validator = libcellml.Validator()
    validator.validateModel(model)
    num_validation_issues = validator.issueCount()
    print(f"The validator has found {num_validation_issues} issues!")
    #      Retrieve the issues, and print their description, url, reference, and
    #      type of item stored to the terminal.  The type of stored item is
    #      available as an enum, which can be turned into a string for output using
    #      the utility function, getItemTypeFromEnum(type).

    for e in range(0, num_validation_issues):
        issue = validator.issue(e)
        reference = issue.referenceHeading()
        print("  Validator issue[{}]:".format(e))
        print("     Description: {}".format(issue.description()))
        # print('     Type of item stored: {}'.format(cellmlElementTypeAsString(issue.item().type())))
        print("     URL: {}".format(issue.url()))
        if reference != "":
            print("    See section {} in the CellML specification.".format(reference))


# <codecell>
if __name__ == "__main__":
    from combine_notebooks import RESULTS_DIR

    cellml_path: Path = RESULTS_DIR / "repressilator.cellml"
    model = create_repressilator(cellml_path)
    validate_model(model)



"""Validation from libsbgn-python.

see https://github.com/matthiaskoenig/libsbgn-python/blob/develop/src/libsbgnpy/validation/validator.py
"""
from libsbgnpy.validation.validator import validate_schematron, validate_xsd

from combine_notebooks import RESULTS_DIR


f_sbgn = str(RESULTS_DIR / "repressilator_sbgn.sbgn")
# f = "error-test-files/PD/pd10101-pass.sbgn"
# f = "../examples/sbgn/glycolysis.sbgn"

xsd_valid = validate_xsd(f_sbgn) is None
print("XSD valid ({}): {}".format(f_sbgn, xsd_valid))



# <markdowncell>
# Create the repressilator step by step.

# See on biomodels.

# https://github.com/sbmlteam/libsbml
# <codecell>
from pathlib import Path

import libsbml

from combine_notebooks.validation.validation_sbml import validate_sbml


# <codecell>


def create_repressilator(sbml_path: Path) -> libsbml.SBMLDocument:
    """Create repressilator."""
    print("Create SBML model")
    # create a new document

    doc: libsbml.SBMLDocument = libsbml.SBMLDocument(3, 2)
    model: libsbml.Model = doc.createModel()

    # --- compartments ---
    # add compartment
    c1: libsbml.Compartment = model.createCompartment()
    c1.setId("cell")
    c1.setSize(1.0)
    c1.setMetaId("meta_cell")
    c1.setSBOTerm("SBO:0000290")
    # set annotation
    c1_cv: libsbml.CVTerm = libsbml.CVTerm()
    c1_cv.setQualifierType(libsbml.BIOLOGICAL_QUALIFIER)
    c1_cv.setBiologicalQualifierType(libsbml.BQB_IS)
    c1_cv.addResource("http://identifiers.org/GO:0005623")
    c1.addCVTerm(c1_cv)

    # --- species ---
    # add species for the 3 mRNAs and 3 Proteins
    s1: libsbml.Species = model.createSpecies()
    s1.setId("PX")
    s1.setMetaId("PX")
    s1.setCompartment("cell")
    s1.setName("LacI protein")
    s1.setSBOTerm("SBO:0000252")
    s1.setInitialAmount(0)
    s1.setHasOnlySubstanceUnits(True)
    # set annotation
    s1_cv: libsbml.CVTerm = libsbml.CVTerm()
    s1_cv.setQualifierType(libsbml.BIOLOGICAL_QUALIFIER)
    s1_cv.setBiologicalQualifierType(libsbml.BQB_IS)
    s1_cv.addResource("http://identifiers.org/uniprot/P03023")
    s1.addCVTerm(s1_cv)

    s2: libsbml.Species = model.createSpecies()
    s2.setId("PY")
    s2.setMetaId("PY")
    s2.setCompartment("cell")
    s2.setName("TetR protein")
    s2.setSBOTerm("SBO:0000252")
    s2.setInitialAmount(0)
    s2.setHasOnlySubstanceUnits(True)
    # set annotation
    s2_cv: libsbml.CVTerm = libsbml.CVTerm()
    s2_cv.setQualifierType(libsbml.BIOLOGICAL_QUALIFIER)
    s2_cv.setBiologicalQualifierType(libsbml.BQB_IS)
    s2_cv.addResource("http://identifiers.org/uniprot/P04483")
    s2.addCVTerm(s2_cv)

    s3: libsbml.Species = model.createSpecies()
    s3.setId("PZ")
    s3.setMetaId("PZ")
    s3.setCompartment("cell")
    s3.setName("cI protein")
    s3.setSBOTerm("SBO:0000252")
    s3.setInitialAmount(0)
    s3.setHasOnlySubstanceUnits(True)
    # set annotation
    s3_cv: libsbml.CVTerm = libsbml.CVTerm()
    s3_cv.setQualifierType(libsbml.BIOLOGICAL_QUALIFIER)
    s3_cv.setBiologicalQualifierType(libsbml.BQB_IS)
    # example setting a model qualifier
    # s3_cv.setQualifierType(libsbml.MODEL_QUALIFIER)
    # s3_cv.setBiologicalQualifierType(libsbml.BQM_IS)
    s3_cv.addResource("http://identifiers.org/uniprot/P03034")
    s3.addCVTerm(s3_cv)

    s4: libsbml.Species = model.createSpecies()
    s4.setId("X")
    s4.setMetaId("meta_X")
    s4.setCompartment("cell")
    s4.setName("LacI mRNA")
    s4.setSBOTerm("SBO:0000250")
    s4.setInitialAmount(0)
    s4.setHasOnlySubstanceUnits(True)
    # set annotation
    s4_cv: libsbml.CVTerm = libsbml.CVTerm()
    s4_cv.setQualifierType(libsbml.BIOLOGICAL_QUALIFIER)
    s4_cv.setBiologicalQualifierType(libsbml.BQB_IS_VERSION_OF)
    s4_cv.addResource("http://identifiers.org/CHEBI:33699")
    s4_cv.setBiologicalQualifierType(libsbml.BQB_IS_VERSION_OF)
    s4_cv.addResource("http://identifiers.org/kegg.compound/C00046")
    s4.addCVTerm(s4_cv)
    s4_cv.setBiologicalQualifierType(libsbml.BQB_ENCODES)
    s4_cv.addResource("http://identifiers.org/uniprot/P03023")
    s4.addCVTerm(s4_cv)

    s5: libsbml.Species = model.createSpecies()
    s5.setId("Y")
    s5.setMetaId("meta_Y")
    s5.setCompartment("cell")
    s5.setName("TetR mRNA")
    s5.setSBOTerm("SBO:0000250")
    s5.setInitialAmount(20)
    s5.setHasOnlySubstanceUnits(True)
    # set annotation
    s5_cv: libsbml.CVTerm = libsbml.CVTerm()
    s5_cv.setQualifierType(libsbml.BIOLOGICAL_QUALIFIER)
    s5_cv.setBiologicalQualifierType(libsbml.BQB_IS_VERSION_OF)
    s5_cv.addResource("http://identifiers.org/CHEBI:33699")
    s5_cv.setBiologicalQualifierType(libsbml.BQB_IS_VERSION_OF)
    s5_cv.addResource("http://identifiers.org/kegg.compound/C00046")
    s5.addCVTerm(s5_cv)
    s5_cv.setBiologicalQualifierType(libsbml.BQB_ENCODES)
    s5_cv.addResource("http://identifiers.org/uniprot/P04483")
    s5.addCVTerm(s5_cv)

    s6: libsbml.Species = model.createSpecies()
    s6.setId("Z")
    s6.setMetaId("meta_Z")
    s6.setCompartment("cell")
    s6.setName("cl mRNA")
    s6.setSBOTerm("SBO:0000250")
    s6.setInitialAmount(0)
    s6.setHasOnlySubstanceUnits(True)
    # set annotation
    s6_cv: libsbml.CVTerm = libsbml.CVTerm()
    s6_cv.setQualifierType(libsbml.BIOLOGICAL_QUALIFIER)
    s6_cv.setBiologicalQualifierType(libsbml.BQB_IS_VERSION_OF)
    s6_cv.addResource("http://identifiers.org/CHEBI:33699")
    s6_cv.setBiologicalQualifierType(libsbml.BQB_IS_VERSION_OF)
    s6_cv.addResource("http://identifiers.org/kegg.compound/C00046")
    s6.addCVTerm(s6_cv)
    s6_cv.setBiologicalQualifierType(libsbml.BQB_ENCODES)
    s6_cv.addResource("http://identifiers.org/uniprot/P03034")
    s6.addCVTerm(s6_cv)

    # --- parameters ---
    # <parameter id="kd_mRNA" constant="false" name="kd_mRNA" sboTerm="SBO:0000356">
    p1: libsbml.Parameter = model.createParameter()
    p1.setId("tau_mRNA")
    p1.setName("mRNA half life")
    p1.setConstant(True)
    p1.setValue(2.0)

    # <parameter id="kd_mRNA" constant="false" name="kd_mRNA" sboTerm="SBO:0000356">
    p2: libsbml.Parameter = model.createParameter()
    p2.setId("kd_mRNA")
    p2.setName("kd_mRNA")
    p2.setSBOTerm("SBO:0000356")
    p2.setConstant(False)

    # <parameter metaid="metaid_0000233" sboTerm="SBO:0000016" id="k_tl" name="k_tl" constant="false">
    p3: libsbml.Parameter = model.createParameter()
    p3.setId("k_tl")
    p3.setName("k_tl")
    p3.setSBOTerm("SBO:0000016")
    p3.setConstant(False)

    # <parameter metaid="metaid_0000025" id="eff" name="translation efficiency" value="20">
    p4: libsbml.Parameter = model.createParameter()
    p4.setId("eff")
    p4.setName("translation efficiency")
    p4.setConstant(True)
    p4.setValue(20)

    # <parameter metaid="metaid_0000032" sboTerm="SBO:0000348" id="t_ave" name="average mRNA life time" constant="false"/>
    p5: libsbml.Parameter = model.createParameter()
    p5.setId("t_ave")
    p5.setName("average mRNA life time")
    p5.setSBOTerm("SBO:0000348")
    p5.setConstant(False)

    # <parameter metaid="metaid_0000133" sboTerm="SBO:0000356" id="kd_prot" name="kd_prot" constant="false">
    p6: libsbml.Parameter = model.createParameter()
    p6.setId("kd_prot")
    p6.setName("kd_prot")
    p6.setSBOTerm("SBO:0000356")
    p6.setConstant(False)

    # <parameter metaid="metaid_0000128" sboTerm="SBO:0000332" id="tau_prot" name="protein half life" value="10"/>
    p7: libsbml.Parameter = model.createParameter()
    p7.setId("tau_prot")
    p7.setName("protien half life")
    p7.setConstant(True)
    p7.setValue(10)

    # <parameter metaid="metaid_0000234" sboTerm="SBO:0000485" id="a0_tr" name="a0_tr" constant="false">
    p8: libsbml.Parameter = model.createParameter()
    p8.setId("a0_tr")
    p8.setName("a0_tr")
    p8.setSBOTerm("SBO:0000485")
    p8.setConstant(False)

    # <parameter metaid="metaid_0500235" sboTerm="SBO:0000485" id="ps_0" name="tps_repr" value="0.0005">
    p9: libsbml.Parameter = model.createParameter()
    p9.setId("ps_0")
    p9.setName("tps_repr")
    p9.setConstant(True)
    p9.setValue(0.0005)

    # <parameter metaid="metaid_0900235" sboTerm="SBO:0000186" id="a_tr" name="a_tr" constant="false">
    p10: libsbml.Parameter = model.createParameter()
    p10.setId("a_tr")
    p10.setName("a_tr")
    p10.setSBOTerm("SBO:0000186")
    p10.setConstant(False)

    # <parameter metaid="metaid_0800235" sboTerm="SBO:0000186" id="ps_a" name="tps_active" value="0.5">
    p11: libsbml.Parameter = model.createParameter()
    p11.setId("ps_a")
    p11.setName("tps_active")
    p11.setConstant(True)
    p11.setValue(0.5)

    # <parameter metaid="metaid_0000027" sboTerm="SBO:0000288" id="KM" name="KM" value="40">
    p12: libsbml.Parameter = model.createParameter()
    p12.setId("KM")
    p12.setName("KM")
    p12.setConstant(True)
    p12.setValue(40)

    # <parameter metaid="metaid_0000026" sboTerm="SBO:0000190" id="n" name="n" value="2">
    p13: libsbml.Parameter = model.createParameter()
    p13.setId("n")
    p13.setName("n")
    p13.setConstant(True)
    p13.setValue(2.0)

    # TODO: a0_tr, a_tr, n, KM

    # --- rules ---
    a1: libsbml.AssignmentRule = model.createAssignmentRule()
    a1.setVariable("kd_mRNA")
    math_ast1: libsbml.ASTNode = libsbml.parseL3FormulaWithModel(
        "ln(2) / tau_mRNA", model
    )
    a1.setMath(math_ast1)

    a2: libsbml.AssignmentRule = model.createAssignmentRule()
    a2.setVariable("t_ave")
    math_ast2: libsbml.ASTNode = libsbml.parseL3FormulaWithModel(
        "tau_mRNA / ln(2)", model
    )
    a2.setMath(math_ast2)

    a3: libsbml.AssignmentRule = model.createAssignmentRule()
    a3.setVariable("k_tl")
    math_ast3: libsbml.ASTNode = libsbml.parseL3FormulaWithModel("eff / t_ave", model)
    a3.setMath(math_ast3)

    a4: libsbml.AssignmentRule = model.createAssignmentRule()
    a4.setVariable("kd_prot")
    math_ast4: libsbml.ASTNode = libsbml.parseL3FormulaWithModel(
        "ln(2) / tau_prot", model
    )
    a4.setMath(math_ast4)

    a5: libsbml.AssignmentRule = model.createAssignmentRule()
    a5.setVariable("a0_tr")
    math_ast5: libsbml.ASTNode = libsbml.parseL3FormulaWithModel("ps_0 * 60", model)
    a5.setMath(math_ast5)

    a6: libsbml.AssignmentRule = model.createAssignmentRule()
    a6.setVariable("a_tr")
    math_ast6: libsbml.ASTNode = libsbml.parseL3FormulaWithModel(
        "(ps_a - ps_0) * 60", model
    )
    a6.setMath(math_ast6)

    # --- reaction ---
    # X -> None
    r1: libsbml.Reaction = model.createReaction()
    r1.setId("Reaction1"),  # set reaction id
    r1.setMetaId("meta_Reaction1")
    r1.setSBOTerm("SBO:0000179")
    r1.setName("degradation of LacI transcripts")
    r1.setReversible(False)
    # set annotation
    r1_cv: libsbml.CVTerm = libsbml.CVTerm()
    r1_cv.setQualifierType(libsbml.BIOLOGICAL_QUALIFIER)
    r1_cv.setBiologicalQualifierType(libsbml.BQB_IS_VERSION_OF)
    r1_cv.addResource("http://identifiers.org/GO:0006402")
    r1.addCVTerm(r1_cv)

    species_ref1: libsbml.SpeciesReference = r1.createReactant()
    species_ref1.setSpecies("X")  # assign reactant species

    math_ast7: libsbml.ASTNode = libsbml.parseL3FormulaWithModel("kd_mRNA * X", model)
    kinetic_law = r1.createKineticLaw()
    kinetic_law.setSBOTerm("SBO:0000049")
    kinetic_law.setMath(math_ast7)

    r2 = model.createReaction()
    r2.setId("Reaction2"),  # set reaction id
    r2.setMetaId("meta_Reaction2")
    r2.setSBOTerm("SBO:0000179")
    r2.setName("degradation of TetR transcripts")
    r2.setReversible(False)

    r2_cv: libsbml.CVTerm = libsbml.CVTerm()
    r2_cv.setQualifierType(libsbml.BIOLOGICAL_QUALIFIER)
    r2_cv.setBiologicalQualifierType(libsbml.BQB_IS_VERSION_OF)
    r2_cv.addResource("http://identifiers.org/GO:0006402")
    r2.addCVTerm(r1_cv)

    species_ref2 = r2.createReactant()
    species_ref2.setSpecies("Y")  # assign reactant species

    math_ast8: libsbml.ASTNode = libsbml.parseL3FormulaWithModel("kd_mRNA * Y", model)
    kinetic_law = r2.createKineticLaw()
    kinetic_law.setSBOTerm("SBO:0000049")
    kinetic_law.setMath(math_ast8)

    r3 = model.createReaction()
    r3.setId("Reaction3"),  # set reaction id
    r3.setMetaId("meta_Reaction3")
    r3.setSBOTerm("SBO:0000179")
    r3.setName("degradation of CI transcripts")
    r3.setReversible(False)
    # set annotation
    r3_cv: libsbml.CVTerm = libsbml.CVTerm()
    r3_cv.setQualifierType(libsbml.BIOLOGICAL_QUALIFIER)
    r3_cv.setBiologicalQualifierType(libsbml.BQB_IS_VERSION_OF)
    r3_cv.addResource("http://identifiers.org/GO:0006402")
    r3.addCVTerm(r3_cv)

    species_ref3 = r3.createReactant()
    species_ref3.setSpecies("Z")  # assign reactant species

    math_ast9: libsbml.ASTNode = libsbml.parseL3FormulaWithModel("kd_mRNA * Z", model)
    kinetic_law = r3.createKineticLaw()
    kinetic_law.setSBOTerm("SBO:0000049")
    kinetic_law.setMath(math_ast9)

    r4 = model.createReaction()
    r4.setId("Reaction4"),  # set reaction id
    r4.setMetaId("meta_Reaction4")
    r4.setSBOTerm("SBO:0000184")
    r4.setName("translation of LacI")
    r4.setReversible(False)
    # set annotation
    r4_cv: libsbml.CVTerm = libsbml.CVTerm()
    r4_cv.setQualifierType(libsbml.BIOLOGICAL_QUALIFIER)
    r4_cv.setBiologicalQualifierType(libsbml.BQB_IS_VERSION_OF)
    r4_cv.addResource("http://identifiers.org/GO:0006412")
    r4.addCVTerm(r4_cv)

    species_ref4 = r4.createProduct()
    species_ref4.setSpecies("PX")  # assign product species

    species_ref_4 = r4.createModifier()
    species_ref_4.setSpecies("X")
    species_ref_4.setSBOTerm("SBO:0000461")

    math_ast10: libsbml.ASTNode = libsbml.parseL3FormulaWithModel("k_tl * X", model)
    kinetic_law = r4.createKineticLaw()
    kinetic_law.setSBOTerm("SBO:0000049")
    kinetic_law.setMath(math_ast10)

    r5 = model.createReaction()
    r5.setId("Reaction5"),  # set reaction id
    r5.setMetaId("meta_Reaction5")
    r5.setSBOTerm("SBO:0000184")
    r5.setName("translation of TetR")
    r5.setReversible(False)
    s3_cv = libsbml.CVTerm()
    # set annotation
    r5_cv: libsbml.CVTerm = libsbml.CVTerm()
    r5_cv.setQualifierType(libsbml.BIOLOGICAL_QUALIFIER)
    r5_cv.setBiologicalQualifierType(libsbml.BQB_IS_VERSION_OF)
    r5_cv.addResource("http://identifiers.org/GO:0006412")
    r5.addCVTerm(r5_cv)

    species_ref5 = r5.createProduct()
    species_ref5.setSpecies("PY")  # assign product species
    species_ref_5 = r5.createModifier()
    species_ref_5.setSpecies("Y")
    species_ref_5.setSBOTerm("SBO:0000461")

    math_ast11: libsbml.ASTNode = libsbml.parseL3FormulaWithModel("k_tl * Y", model)
    kinetic_law = r5.createKineticLaw()
    kinetic_law.setSBOTerm("SBO:0000049")
    kinetic_law.setMath(math_ast11)

    r6 = model.createReaction()
    r6.setId("Reaction6"),  # set reaction id
    r6.setMetaId("meta_Reaction6")
    r6.setSBOTerm("SBO:0000184")
    r6.setName("translation of CI")
    r6.setReversible(False)
    # set annotation
    r6_cv: libsbml.CVTerm = libsbml.CVTerm()
    r6_cv.setQualifierType(libsbml.BIOLOGICAL_QUALIFIER)
    r6_cv.setBiologicalQualifierType(libsbml.BQB_IS_VERSION_OF)
    r6_cv.addResource("http://identifiers.org/GO:0006412")
    r6.addCVTerm(r6_cv)

    species_ref6 = r6.createProduct()
    species_ref6.setSpecies("PZ")  # assign product species
    species_ref_6 = r6.createModifier()
    species_ref_6.setSpecies("Z")
    species_ref_6.setSBOTerm("SBO:0000461")

    math_ast12: libsbml.ASTNode = libsbml.parseL3FormulaWithModel("k_tl * Z", model)
    kinetic_law = r6.createKineticLaw()
    kinetic_law.setSBOTerm("SBO:0000049")
    kinetic_law.setMath(math_ast12)

    r7 = model.createReaction()
    r7.setId("Reaction7"),  # set reaction id
    r7.setMetaId("meta_Reaction7")
    r7.setSBOTerm("SBO:0000179")
    r7.setName("degradation of LacI")
    r7.setReversible(False)
    # set annotation
    r7_cv: libsbml.CVTerm = libsbml.CVTerm()
    r7_cv.setQualifierType(libsbml.BIOLOGICAL_QUALIFIER)
    r7_cv.setBiologicalQualifierType(libsbml.BQB_IS_VERSION_OF)
    r7_cv.addResource("http://identifiers.org/GO:0030163")
    r7.addCVTerm(r7_cv)

    species_ref7 = r7.createReactant()
    species_ref7.setSpecies("PX")  # assign reactant species

    math_ast13: libsbml.ASTNode = libsbml.parseL3FormulaWithModel("kd_prot * PX", model)
    kinetic_law = r7.createKineticLaw()
    kinetic_law.setSBOTerm("SBO:0000049")
    kinetic_law.setMath(math_ast13)

    r8 = model.createReaction()
    r8.setId("Reaction8"),  # set reaction id
    r8.setMetaId("meta_Reaction8")
    r8.setSBOTerm("SBO:0000179")
    r8.setName("degradation of TetR")
    r8.setReversible(False)
    # set annotation
    r8_cv: libsbml.CVTerm = libsbml.CVTerm()
    r8_cv.setQualifierType(libsbml.BIOLOGICAL_QUALIFIER)
    r8_cv.setBiologicalQualifierType(libsbml.BQB_IS_VERSION_OF)
    r8_cv.addResource("http://identifiers.org/GO:0030163")
    r8.addCVTerm(r8_cv)

    species_ref8 = r8.createReactant()
    species_ref8.setSpecies("PY")  # assign reactant species

    math_ast14: libsbml.ASTNode = libsbml.parseL3FormulaWithModel("kd_prot * PY", model)
    kinetic_law = r8.createKineticLaw()
    kinetic_law.setSBOTerm("SBO:0000049")
    kinetic_law.setMath(math_ast14)

    r9 = model.createReaction()
    r9.setId("Reaction9"),  # set reaction id
    r9.setMetaId("meta_Reaction9")
    r9.setSBOTerm("SBO:0000179")
    r9.setName("degradation of CI")
    r9.setReversible(False)
    # set annotation
    r9_cv: libsbml.CVTerm = libsbml.CVTerm()
    r9_cv.setQualifierType(libsbml.BIOLOGICAL_QUALIFIER)
    r9_cv.setBiologicalQualifierType(libsbml.BQB_IS_VERSION_OF)
    r9_cv.addResource("http://identifiers.org/GO:0030163")
    r9.addCVTerm(r9_cv)

    species_ref9 = r9.createReactant()
    species_ref9.setSpecies("PZ")  # assign reactant species

    math_ast15: libsbml.ASTNode = libsbml.parseL3FormulaWithModel("kd_prot * PZ", model)
    kinetic_law = r9.createKineticLaw()
    kinetic_law.setSBOTerm("SBO:0000049")
    kinetic_law.setMath(math_ast15)

    r10 = model.createReaction()
    r10.setId("Reaction10"),  # set reaction id
    r10.setMetaId("meta_Reaction10")
    r10.setSBOTerm("SBO:0000183")
    r10.setName("transcription of LacI")
    r10.setReversible(False)
    # set annotation
    r10_cv: libsbml.CVTerm = libsbml.CVTerm()
    r10_cv.setQualifierType(libsbml.BIOLOGICAL_QUALIFIER)
    r10_cv.setBiologicalQualifierType(libsbml.BQB_IS_VERSION_OF)
    r10_cv.addResource("http://identifiers.org/GO:0006351")
    r10.addCVTerm(r10_cv)

    species_ref10 = r10.createProduct()
    species_ref10.setSpecies("X")  # assign product species
    species_ref_10 = r10.createModifier()
    species_ref_10.setSpecies("PZ")
    species_ref_10.setSBOTerm("SBO:0000536")

    math_ast16: libsbml.ASTNode = libsbml.parseL3FormulaWithModel(
        "a0_tr + (a_tr * power(KM, n)) / (power(KM, n) + power(PZ, n))", model
    )
    kinetic_law = r10.createKineticLaw()
    kinetic_law.setMath(math_ast16)

    r11 = model.createReaction()
    r11.setId("Reaction11"),  # set reaction id
    r11.setMetaId("meta_Reaction11")
    r11.setSBOTerm("SBO:0000183")
    r11.setName("transcription of TetR")
    r11.setReversible(False)
    # set annotation
    r11_cv: libsbml.CVTerm = libsbml.CVTerm()
    r11_cv.setQualifierType(libsbml.BIOLOGICAL_QUALIFIER)
    r11_cv.setBiologicalQualifierType(libsbml.BQB_IS_VERSION_OF)
    r11_cv.addResource("http://identifiers.org/GO:0006351")
    r11.addCVTerm(r11_cv)

    species_ref11 = r11.createProduct()
    species_ref11.setSpecies("Y")  # assign product species
    species_ref_11 = r11.createModifier()
    species_ref_11.setSpecies("PX")
    species_ref_11.setSBOTerm("SBO:0000536")

    math_ast17: libsbml.ASTNode = libsbml.parseL3FormulaWithModel(
        "a0_tr + (a_tr * power(KM, n)) / (power(KM, n) + power(PX, n))", model
    )
    kinetic_law = r11.createKineticLaw()
    kinetic_law.setMath(math_ast17)

    r12 = model.createReaction()
    r12.setId("Reaction12"),  # set reaction id
    r12.setMetaId("meta_Reaction12")
    r12.setSBOTerm("SBO:0000183")
    r12.setName("transcription of CI")
    r12.setReversible(False)
    # set annotation
    r12_cv: libsbml.CVTerm = libsbml.CVTerm()
    r12_cv.setQualifierType(libsbml.BIOLOGICAL_QUALIFIER)
    r12_cv.setBiologicalQualifierType(libsbml.BQB_IS_VERSION_OF)
    r12_cv.addResource("http://identifiers.org/GO:0006351")
    r12.addCVTerm(r12_cv)

    species_ref12 = r12.createProduct()
    species_ref12.setSpecies("Z")  # assign product species
    species_ref_12 = r12.createModifier()
    species_ref_12.setSpecies("PY")
    species_ref_12.setSBOTerm("SBO:0000536")

    math_ast18: libsbml.ASTNode = libsbml.parseL3FormulaWithModel(
        "a0_tr + (a_tr * power(KM, n)) / (power(KM, n) + power(PY, n))", model
    )
    kinetic_law = r12.createKineticLaw()
    kinetic_law.setMath(math_ast18)

    print("-" * 80)
    print(libsbml.writeSBMLToString(doc))
    print("-" * 80)

    # write to file
    libsbml.writeSBMLToFile(doc, str(sbml_path))

    # validate file
    validate_sbml(doc, units_consistency=False)

    return doc


# <codecell>
if __name__ == "__main__":
    from combine_notebooks import RESULTS_DIR

    # RESOURCES_DIR: Path = Path(__file__).parent / "resources"
    # RESULTS_DIR: Path = RESOURCES_DIR / "results"
    doc: libsbml.SBMLDocument = create_repressilator(
        sbml_path=RESULTS_DIR / "repressilator_libsbml.xml"
    )



# <markdowncell>

# Example for creating a COMBINE archive.

# <codecell>
from pathlib import Path

from pymetadata.console import console
from pymetadata.omex import EntryFormat, ManifestEntry, Omex


# <markdowncell>

# Creating an empty archive and adding entry for SBML.


# <codecell>


def create_omex(omex_path: Path, results_dir: Path) -> None:
    """Create OMEX archive of resources."""
    omex = Omex()
    omex.add_entry(
        entry=ManifestEntry(
            location="./sbml/repressilator_sbml_libsbml.xml",
            format=EntryFormat.SBML_L3V2,
            master=False,
        ),
        entry_path=results_dir / "repressilator_sbml_libsbml.xml",
    )

    omex.add_entry(
        entry=ManifestEntry(
            location="./sbml/repressilator_sbml_antimony.xml",
            format=EntryFormat.SBML_L3V2,
            master=False,
        ),
        entry_path=results_dir / "repressilator_sbml_antimony.xml",
    )

    omex.add_entry(
        entry=ManifestEntry(
            location="./sbml/repressilator_sbml_sbmlutils.xml",
            format=EntryFormat.SBML_L3V2,
            master=False,
        ),
        entry_path=results_dir / "repressilator_sbml_sbmlutils.xml",
    )

    omex.add_entry(
        entry=ManifestEntry(
            location="./cellml/repressilator_cellml.cellml",
            format=EntryFormat.CELLML,
            master=False,
        ),
        entry_path=results_dir / "repressilator_cellml.cellml",
    )

    omex.add_entry(
        entry=ManifestEntry(
            location="./biopax/repressilator_biopax.owl",
            format=EntryFormat.BIOPAX,
            master=False,
        ),
        entry_path=results_dir / "repressilator_biopax.owl",
    )

    omex.add_entry(
        entry=ManifestEntry(
            location="./sbgn/repressilator_sbgn.sbgn",
            format=EntryFormat.SBGN,
            master=False,
        ),
        entry_path=results_dir / "repressilator_sbgn.sbgn",
    )
    omex.add_entry(
        entry=ManifestEntry(
            location="./sbgn/repressilator_sbgn.png",
            format=EntryFormat.PNG,
            master=False,
        ),
        entry_path=results_dir / "repressilator_sbgn.png",
    )

    omex.add_entry(
        entry=ManifestEntry(
            location="./sedml/repressilator_sedml.xml",
            format=EntryFormat.SEDML,
            master=False,
        ),
        entry_path=results_dir / "repressilator_sedml.xml"
        # FIXME: add the model relative and test
    )

    console.print(omex)
    omex.to_omex(omex_path)


if __name__ == "__main__":
    from combine_notebooks import RESULTS_DIR

    create_omex(omex_path=RESULTS_DIR / "combine.omex", results_dir=RESULTS_DIR)



"""
https://github.com/combine-org/combine-notebooks/issues/5
"""
import getpass

import loica as lc
import sbol3
from flapjack import Flapjack
from sbol_utilities import component


# %matplotlib inline

dna = fj.get("dna", name="Rep")
if len(dna) == 0:
    dna = fj.create("dna", name="Rep")
vector = fj.get("vector", name="Rep")
if len(vector) == 0:
    vector = fj.create("vector", name="Rep", dnas=dna.id)
study = fj.get("study", name="Loica testing")
if len(study) == 0:
    study = fj.create("study", name="Loica testing", description="Test")
media = fj.get("media", name="Loica")
if len(media) == 0:
    media = fj.create("media", name="Loica", description="Simulated loica media")
strain = fj.get("strain", name="Loica strain")
if len(strain) == 0:
    strain = fj.create("strain", name="Loica strain", description="Loica test strain")

cfp = fj.get("signal", name="CFP")
if len(cfp) == 0:
    cfp = fj.create(
        "signal",
        name="CFP",
        color="cyan",
        description="Simulated cyan fluorescent protein",
    )
yfp = fj.get("signal", name="YFP")
if len(yfp) == 0:
    yfp = fj.create(
        "signal",
        name="YFP",
        color="yellow",
        description="Simulated yellow fluorescent protein",
    )
rfp = fj.get("signal", name="RFP")
if len(rfp) == 0:
    rfp = fj.create(
        "signal",
        name="RFP",
        color="red",
        description="Simulated red fluorescent protein",
    )

biomass_signal = fj.get("signal", name="OD")


def growth_rate(t):
    return gompertz_growth_rate(t, 0.01, 1, 1, 1)


def biomass(t):
    return gompertz(t, 0.01, 1, 1, 1)


doc = sbol3.Document()
sbol3.set_namespace("https://github.com/Gonza10V")

ptet, ptet_seq = component.promoter(
    "L3S2P21_UPA20_pTetR_RiboJ",
    "GTCCCTCGGTACCAAATTCCAGAAAAGAGGCCTCCCGAAAGGGGGGCCTTTTTTCGTTTTGGTCCGTGCCTACTCTGGAAAATCTtccctatcagtgatagagattgacatccctatcagtgatagagatactgagcacatcagcaggacgcactgaccAGCTGTCACCGGATGTGCTTTCCGGTCTGATGAGTCCGTGAGGACGAAACAGCCTCTACAAATAATTTTGTTTAAGGCTCG",
    description="BASIC - Potvin pTet (based on B-P27-pTetR-F1)",
)
plac, plac_seq = component.promoter(
    "L3S2P11_UPA20_pLac_RiboJ10",
    "GTCCCTCGGTACCAAATTCCAGAAAAGAGACGCTTTCGAGCGTCTTTTTTCGTTTTGGTCCGTGCCTACTCTGGAAAATCTcctttcgtcttcacctcgagaattgtgagcggataacaattgacattgtgagcggataacaagatactgagcacatcagcaggacgcactgaccgaattcattAGCGCTCAACGGGTGTGCTTCCCGTTCTGATGAGTCCGTGAGGACGAAAGCGCCTCTACAAATAATTTTGTTTAAGGCTCG",
    description="BASIC - Potvin pLac (based on B-P39-pLac-F2)",
)
plam, plam_seq = component.promoter(
    "L3S1P13_UPA20_plambda_RiboJ51",
    "GTCCGACGAACAATAAGGCCTCCCTAACGGGGGGCCTTTTTTATTGATAACAAAAGTGCCTACTCTGGAAAATCTccgccgccctagacctagctgcaggtcgaggataaatatctaacaccgtgcgtgttgactattttacctctggcggtgataatggttgcatgtactagaattcattAGTAGTCACCGGCTGTGCTTGCCGGTCTGATGAGCCTGTGAAGGCGAAACTACCTCTACAAATAATTTTGTTTAAGGCTCG",
    description="BASIC - Potvin plambda (based on B-P41-pPhlF-F3)",
)

op_ptet = component.engineered_region(
    "operator_ptet", [ptet], description="LOICA Operator pTet"
)
op_plac = component.engineered_region(
    "operator_plac", [plac], description="LOICA Operator pLac"
)
op_plam = component.engineered_region(
    "operator_plam", [plam], description="LOICA Operator pLambda"
)

toplevels = [ptet, ptet_seq, plac, plac_seq, plam, plam_seq, op_ptet, op_plac, op_plam]
doc.add(toplevels)

operators = [op_ptet, op_plac, op_plam]

for obj in doc.objects:
    print(obj.identity)
report_sbol3 = doc.validate()
print(len(report_sbol3))

rbs1, rbs1_seq = component.rbs(
    "RBS1",
    "ttgaacaccgtcTCAGGTAAGTATCAGTTGTAAatcacacaggacta",
    description="BASIC Linker RBS1",
)
rbs2, rbs2_seq = component.rbs(
    "RBS2",
    "ttgaacaccgtcTCAGGTAAGTATCAGTTGTAAaaagaggggaaata",
    description="BASIC Linker RBS2",
)
rbs3, rbs3_seq = component.rbs(
    "RBS3",
    "ttgaacaccgtcTCAGGTAAGTATCAGTTGTAAaaagaggagaaata",
    description="BASIC Linker RBS3",
)

mven, mven_seq = component.cds(
    "mVenus",
    "GTCCCTCGGTACCAAATTCCAGAAAAGAGGCCTCCCGAAAGGGGGGCCTTTTTTCGTTTTGGTCCGTGCCTACTCTGGAAAATCTtccctatcagtgatagagattgacatccctatcagtgatagagatactgagcacatcagcaggacgcactgaccAGCTGTCACCGGATGTGCTTTCCGGTCTGATGAGTCCGTGAGGACGAAACAGCCTCTACAAATAATTTTGTTTAAGGCTCG",
    description="mVenus Coding Sequence from Potvin-Trottier pLPT119, no BasI site, no stop codon",
)
tetr, tetr_seq = component.cds(
    "TetR",
    "GTCCatgtccagattagataaaagtaaagtgattaacagcgcattagagctgcttaatgaggtcggaatcgaaggtttaacaacccgtaaactcgcccagaagctaggtgtagagcagcctacattgtattggcatgtaaaaaataagcgggctttgctcgacgccttagccattgagatgttagataggcaccatactcacttttgccctttagaaggggaaagctggcaagattttttacgtaataacgctaaaagttttagatgtgctttactaagtcatcgcgatggagcaaaagtacatttaggtacacggcctacagaaaaacagtatgaaactctcgaaaatcaattagcctttttatgccaacaaggtttttcactagagaatgcattatatgcactcagcgctgtggggcattttactttaggttgcgtattggaagatcaagagcatcaagtcgctaaagaagaaagggaaacacctactactgatagtatgccgccattattacgacaagctatcgaattatttgatcaccaaggtgcagagccagccttcttattcggccttgaattgatcatatgcggattagaaaaacaacttaaatgtgaaagtgggtctGGCTCG",
    description="TetR Coding Sequence from Potvin-Trottier pLPT119, no stop codon",
)
laci, laci_seq = component.cds(
    "LacI",
    "GTCCatggtgaatgtgaaaccagtaacgttatacgatgtcgcagagtatgccggtgtctcttatcagaccgtttcccgcgtggtgaaccaggccagccacgtttctgcgaaaacgcgggaaaaagtggaagcggcgatggcggagctgaattacattcccaaccgcgtggcacaacaactggcgggcaaacagtcgttgctgattggcgttgccacctccagtctggccctgcacgcgccgtcgcaaattgtcgcggcgattaaatctcgcgccgatcaactgggtgccagcgtggtggtgtcgatggtagaacgaagcggcgtcgaagcctgtaaagcggcggtgcacaatcttctcgcgcaacgcgtcagtgggctgatcattaactatccgctggatgaccaggatgccattgctgtggaagctgcctgcactaatgttccggcgttatttcttgatgtctctgaccagacacccatcaacagtattattttctcccatgaagacggtacgcgactgggcgtggagcatctggtcgcattgggtcaccagcaaatcgcgctgttagcgggcccattaagttctgtctcggcgcgtctgcgtctggctggctggcataaatatctcactcgcaatcaaattcagccgatagcggaacgggaaggcgactggagtgccatgtccggttttcaacaaaccatgcaaatgctgaatgagggcatcgttcccactgcgatgctggttgccaacgatcagatggcgctgggcgcaatgcgcgccattaccgagtccgggctgcgcgttggtgcggatatctcggtagtgggatacgacgataccgaagacagctcatgttatatcccgccgttaaccaccatcaaacaggattttcgcctgctggggcaaaccagcgtggaccgcttgctgcaactctctcagggccaggcggtgaagggcaatcagctgttgcccgtctcactggtgaaaagaaaaaccaccctggcgcccaatacgcaaaccgcctctccccgcgcgttggccgattcattaatgcagctggcacgacaggtttcccgactggaaagcgggcagGGCTCG",
    description="LacI Coding Sequence from Potvin-Trottier pLPT119, no stop codon",
)
lamr, lamr_seq = component.cds(
    "LamR",
    "GTCCCTCGGTACCAAATTCCAGAAAAGAGGCCTCCCGAAAGGGGGGCCTTTTTTCGTTTTGGTCCGTGCCTACTCTGGAAAATCTtccctatcagtgatagagattgacatccctatcagtgatagagatactgagcacatcagcaggacgcactgaccAGCTGTCACCGGATGTGCTTTCCGGTCTGATGAGTCCGTGAGGACGAAACAGCCTCTACAAATAATTTTGTTTAAGGCTCG",
    description="lambdaR Coding Sequence from Potvin-Trottier pLPT119, no stop codon",
)

m0050, m0050_seq = component.protein_stability_element(
    "M0050",
    "gctgctaacgacgaaaactacgctctggctgctTAAattgaacta",
    description="http://parts.igem.org/wiki/index.php?title=Part:BBa_M0050",
)
m0051, m0051_seq = component.protein_stability_element(
    "M0051",
    "gctgctaacgacgaaaactacaactacgctgacgcttctTAActa",
    description="http://parts.igem.org/wiki/index.php?title=Part:BBa_M0051",
)
m0052, m0052_seq = component.protein_stability_element(
    "M0052",
    "gctgctaacgacgaaaactacgctgacgcttctTAAattgaacta",
    description="http://parts.igem.org/wiki/index.php?title=Part:BBa_M0052",
)

ter1, ter1_seq = component.terminator(
    "TER1",
    "GTCCatttgtcctactcaggagagcgttcaccgacaaacaacagataaaacgaaaggcccagtctttcgactgagcctttcgttttatttgTAAGGCTCG",
    description="rrnB T1 terminator from Potvin-Trottier pLPT119, extra stop codon",
)

toplevels = [
    rbs1,
    rbs1_seq,
    rbs2,
    rbs2_seq,
    rbs3,
    rbs3_seq,
    mven,
    mven_seq,
    tetr,
    tetr_seq,
    laci,
    laci_seq,
    lamr,
    lamr_seq,
    m0050,
    m0050_seq,
    m0051,
    m0051_seq,
    m0052,
    m0052_seq,
    ter1,
    ter1_seq,
]
doc.add(toplevels)

# Automate design from lists of components, could be a collection
rbss = [rbs1]  # [rbs1, rbs2, rbs3]
cdss = [mven, tetr, laci, lamr]
degtags = [m0050]  # [m0050, m0051, m0052]
terminators = [ter1]

# Wrap it together
geneproducts = []
for rbs in rbss:
    for cds in cdss:
        for degtag in degtags:
            for ter in terminators:
                geneproduct = component.engineered_region(
                    f"geneproduct_{cds.display_id}_{degtag.display_id}_{ter.display_id}",
                    [rbs, cds, degtag, ter],
                    description="LOICA gene product",
                )
                doc.add(geneproduct)
                geneproducts.append(geneproduct)

for obj in doc.objects:
    print(obj.identity)
report_sbol3 = doc.validate()
print(len(report_sbol3))

rep = lc.GeneticNetwork(vector=vector.id[0])

tetr_reg = lc.Regulator(name="TetR", degradation_rate=1, sbol_comp=geneproducts[1])
laci_reg = lc.Regulator(
    name="LacI", degradation_rate=1, init_concentration=5, sbol_comp=geneproducts[2]
)
ci_reg = lc.Regulator(name="cI", degradation_rate=1, sbol_comp=geneproducts[3])
rep.add_regulator([tetr_reg, laci_reg, ci_reg])

mven_rep = lc.Reporter(
    name="mVenus",
    degradation_rate=1,
    signal_id=yfp.id[0],
    sbol_comp=geneproducts[0],
    color="yellow",
)
rep.add_reporter(mven_rep)

laci_not_tetr = lc.Hill1(
    input=laci_reg, output=tetr_reg, alpha=[100, 0], K=1, n=2, sbol_comp=op_plac
)
ci_not_laci = lc.Hill1(
    input=ci_reg, output=laci_reg, alpha=[100, 0], K=1, n=2, sbol_comp=op_plam
)
tetr_not_ci = lc.Hill1(
    input=tetr_reg, output=ci_reg, alpha=[100, 0], K=1, n=2, sbol_comp=op_ptet
)
tetr_not_mven = lc.Hill1(
    input=tetr_reg, output=mven_rep, alpha=[100, 0], K=1, n=2, sbol_comp=op_ptet
)
rep.add_operator([laci_not_tetr, ci_not_laci, tetr_not_ci, tetr_not_mven])
rep.draw()

repressilator_doc = rep.to_sbol(sbol_doc=doc)
for obj in repressilator_doc.objects:
    print(obj.identity)
report_sbol3 = repressilator_doc.validate()
print(len(report_sbol3))



# <markdowncell>
# Biological Pathway Exchange (BioPAX) is a standard language that aims to enable integration, exchange, visualization and analysis of biological pathway data. Specifically, BioPAX supports data exchange between pathway data groups and thus reduces the complexity of interchange between data formats by providing an accepted standard format for pathway data. It is an open and collaborative effort by the community of researchers, software developers, and institutions. BioPAX is defined in OWL DL and is represented in the RDF/XML format. For more details, see Demir E et al. 2010. The BioPAX community standard for pathway data sharing, Nature Biotechnology. 28(9).
# <markdowncell>
# http://www.biopax.org/
# Specification: http://www.biopax.org/release/biopax-level3-documentation.pdf
# https://pypi.org/project/pybiopax/
# https://github.com/indralab/pybiopax
# <markdowncell>
# For the reactions you need probably the `BiochemicalReaction` and `Degradation`.
# For the PhysicalEntities you would use `Protein` and `RNA`.
# For the inhibition you will probably use `Modulation`.
# Have a look at the specification and the introduction to BioPax.
# <markdowncell>
# Example for fetching information
# https://github.com/indralab/pybiopax/blob/master/notebooks/tutorial.ipynb

# <codecell>

from pathlib import Path

import pybiopax
from pybiopax.api import model_to_owl_str
from pybiopax.biopax import (
    BioSource,
    CellularLocationVocabulary,
    Control,
    PhysicalEntity,
    Rna,
    UnificationXref,
)
from pybiopax.biopax.base import Protein
from pybiopax.biopax.model import (
    BiochemicalReaction,
    BioPaxModel,
    Provenance,
    PublicationXref,
    Stoichiometry,
)


# <codecell>
def create_repressilator(biopax_path: Path) -> BioPaxModel:
    """Create repressilator using biopax."""
    objects = []

    # <bp:Protein rdf:about="PX">
    #  <bp:xref rdf:resource="http://identifiers.org/uniprot/P03023" />
    #  <bp:displayName rdf:datatype = "http://www.w3.org/2001/XMLSchema#string">LacI protein</bp:displayName>
    # </bp:Protein>
    # Protein(LacI protein)
    # {'_controller_of': set(), 'xref': [<pybiopax.biopax.util.UnificationXref object at 0x7fe1eb423160>],
    # 'display_name': 'LacI protein', 'standard_name': None, '_name': [], 'evidence': [], 'uid': 'PX', 'comment': [], 'availability': None, 'data_source': [], '_participant_of': set(), 'feature': [], 'not_feature': [], 'member_physical_entity': [], 'cellular_location': None, '_component_of': set(), '_member_physical_entity_of': {PhysicalEntity(PX), PhysicalEntity(PX), PhysicalEntity(PX)}, 'entity_reference': None}
    #
    # <bp:PhysicalEntity rdf:about="RIGHT_0_conversion_Reaction5_PY">
    #  <bp:cellularLocation rdf:resource="cell" />
    #  <bp:displayName rdf:datatype = "http://www.w3.org/2001/XMLSchema#string">PY</bp:displayName>
    #  <bp:memberPhysicalEntity rdf:resource="PY" />
    # </bp:PhysicalEntity>

    p1 = PhysicalEntity(
        uid="RIGHT_0_conversion_Reaction5_PY",
        display_name="PY",
        cellular_location="cell",
        member_physical_entity="PY",
    )
    objects.append(p1)

    # <bp:Stoichiometry rdf:about="LEFT_0_conversion_Reaction2_Y_STOICHIOMETRY">
    #  <bp:physicalEntity rdf:resource="LEFT_0_conversion_Reaction2_Y" />
    #  <bp:stoichiometricCoefficient rdf:datatype = "http://www.w3.org/2001/XMLSchema#float">1.0</bp:stoichiometricCoefficient>
    # </bp:Stoichiometry>
    s1 = Stoichiometry(
        uid="LEFT_0_conversion_Reaction2_Y_STOICHIOMETRY",
        physical_entity="LEFT_0_conversion_Reaction2_Y",
        stoichiometric_coefficient="1",
    )
    objects.append(s1)

    # '''
    # <bp:Provenance rdf:about="datasource_1">
    #  <bp:xref rdf:resource="http://identifiers.org/biomodels.db/BIOMD0000000012" />
    #  <bp:displayName rdf:datatype = "http://www.w3.org/2001/XMLSchema#string">BioModels Database</bp:displayName>
    # </bp:Provenance>
    # '''
    p3 = Provenance(
        uid="datasource_1",
        xref="http://identifiers.org/biomodels.db/BIOMD0000000012",
        display_name="BioModels Database",
    )
    objects.append(p3)
    #
    p4 = PublicationXref(
        uid="http://identifiers.org/pubmed/10659856", id="10659856", db="PubMed"
    )
    objects.append(p4)

    p5 = Provenance(
        uid="datasource_2",
        xref="http://identifiers.org/biomodels.db/MODEL6615351360",
        display_name="BioModels Database",
    )
    objects.append(p5)

    u1 = UnificationXref(
        uid="http://identifiers.org/biomodels.db/BIOMD0000000012",
        id="BIOMD0000000012",
        db="BioModels Database",
    )
    objects.append(u1)

    u2 = UnificationXref(
        uid="http://identifiers.org/obo.go/GO:0006402",
        id="GO:0006402",
        db="Gene Ontology",
    )
    objects.append(u2)

    s2 = Stoichiometry(
        uid="LEFT_0_conversion_Reaction7_PX_STOICHIOMETRY",
        physical_entity="LEFT_0_conversion_Reaction7_PX",
        stoichiometric_coefficient="1",
    )
    objects.append(s2)

    #
    # '''
    # <bp:Control rdf:about="control_Reaction4_X_0">
    #  <bp:controlled rdf:resource="conversion_Reaction4" />
    #  <bp:controlType rdf:datatype = "http://www.w3.org/2001/XMLSchema#string">INHIBITION</bp:controlType>
    #  <bp:controller rdf:resource="PEP0_control_Reaction4X" />
    # </bp:Control>
    # '''
    c1 = Control(
        uid="control_Reaction4_X_0",
        controlled="conversion_Reaction4",
        control_type="INHIBITION",
        controller="PEP0_control_Reaction4X",
    )
    objects.append(c1)

    s2 = Stoichiometry(
        uid="RIGHT_0_conversion_Reaction6_PZ_STOICHIOMETRY",
        physical_entity="RIGHT_0_conversion_Reaction6_PZ",
        stoichiometric_coefficient="1",
    )
    objects.append(s2)

    c2 = PhysicalEntity(
        uid="PEP0_control_Reaction4X",
        cellular_location="cell",
        display_name="X",
        member_physical_entity="X",
    )
    objects.append(c2)

    u3 = UnificationXref(
        uid="http://identifiers.org/obo.go/GO:0030163",
        id="GO:0030163",
        db="Gene Ontology",
    )
    objects.append(u3)

    s3 = Stoichiometry(
        uid="RIGHT_0_conversion_Reaction12_Z_STOICHIOMETRY",
        physical_entity="RIGHT_0_conversion_Reaction12_Z",
        stoichiometric_coefficient="1",
    )
    objects.append(s3)

    c3 = PhysicalEntity(
        uid="LEFT_0_conversion_Reaction7_PX",
        cellular_location="cell",
        display_name="PX",
        member_physical_entity="PX",
    )
    objects.append(c3)

    s4 = Stoichiometry(
        uid="RIGHT_0_conversion_Reaction11_Y_STOICHIOMETRY",
        physical_entity="RIGHT_0_conversion_Reaction11_Y",
        stoichiometric_coefficient="1",
    )
    objects.append(s4)

    c4 = CellularLocationVocabulary(
        uid="cell", term="cell", xref="http://identifiers.org/obo.go/GO:0005623"
    )
    objects.append(c4)

    b1 = BiochemicalReaction(
        uid="conversion_Reaction12",
        xref="http://identifiers.org/obo.go/GO:0006351",
        participant_stoichiometry="RIGHT_0_conversion_Reaction12_Z_STOICHIOMETRY",
        display_name="transcription of CI",
        right="RIGHT_0_conversion_Reaction12_Z",
    )
    objects.append(b1)

    b1 = BiochemicalReaction(
        uid="conversion_Reaction11",
        xref="http://identifiers.org/obo.go/GO:0006351",
        participant_stoichiometry="RIGHT_0_conversion_Reaction11_Y_STOICHIOMETRY",
        display_name="transcription of TetR",
        right="RIGHT_0_conversion_Reaction11_Y",
    )
    objects.append(b1)

    b1 = BiochemicalReaction(
        uid="conversion_Reaction10",
        xref="http://identifiers.org/obo.go/GO:0006351",
        participant_stoichiometry="RIGHT_0_conversion_Reaction10_X_STOICHIOMETRY",
        display_name="transcription of LacI",
        right="RIGHT_0_conversion_Reaction10_X",
    )
    objects.append(b1)

    p1 = PhysicalEntity(
        uid="LEFT_0_conversion_Reaction9_PZ",
        cellular_location="cell",
        display_name="PZ",
        member_physical_entity="PZ",
    )
    objects.append(p1)

    u1 = UnificationXref(
        uid="http://identifiers.org/kegg.compound/C00046",
        id="C00046",
        db="KEGG Compound",
    )
    objects.append(u1)

    u1 = UnificationXref(
        uid="http://identifiers.org/obo.go/GO:0005623",
        id="GO:0005623",
        db="Gene Ontology",
    )
    objects.append(u1)

    p1 = PhysicalEntity(
        uid="LEFT_0_conversion_Reaction8_PY",
        cellular_location="cell",
        display_name="PY",
        member_physical_entity="PY",
    )
    objects.append(p1)

    u1 = UnificationXref(
        uid="http://identifiers.org/chebi/CHEBI:33699", id="CHEBI:33699", db="ChEBI"
    )
    objects.append(u1)

    p1 = Protein(
        uid="PX",
        xref="http://identifiers.org/uniprot/P03023",
        display_name="LacI protein",
    )
    objects.append(p1)

    p1 = Protein(
        uid="PY",
        xref="http://identifiers.org/uniprot/P04483",
        display_name="TetR protein",
    )
    objects.append(p1)

    s1 = Stoichiometry(
        uid="RIGHT_0_conversion_Reaction10_X_STOICHIOMETRY",
        physical_entity="RIGHT_0_conversion_Reaction10_X",
        stoichiometric_coefficient="1",
    )
    objects.append(s1)

    p1 = Protein(
        uid="PZ",
        xref="http://identifiers.org/uniprot/P03034",
        display_name="cI protein",
    )
    objects.append(p1)

    c1 = BiochemicalReaction(
        uid="conversion_Reaction2",
        xref="http://identifiers.org/obo.go/GO:0006402",
        participant_stoichiometry="LEFT_0_conversion_Reaction2_Y_STOICHIOMETRY",
        left="LEFT_0_conversion_Reaction2_Y",
        display_name="degradation of TetR transcripts",
    )
    objects.append(c1)

    s1 = Stoichiometry(
        uid="RIGHT_0_conversion_Reaction4_PX_STOICHIOMETRY",
        physical_entity="RIGHT_0_conversion_Reaction4_PX",
        stoichiometric_coefficient="1",
    )
    objects.append(s1)

    p1 = PhysicalEntity(
        uid="LEFT_0_conversion_Reaction3_Z",
        cellular_location="cell",
        display_name="Z",
        member_physical_entity="Z",
    )
    objects.append(p1)

    c1 = BiochemicalReaction(
        uid="conversion_Reaction3",
        xref="http://identifiers.org/obo.go/GO:0006402",
        participant_stoichiometry="LEFT_0_conversion_Reaction3_Z_STOICHIOMETRY",
        left="LEFT_0_conversion_Reaction3_Z",
        display_name="degradation of CI transcripts",
    )
    objects.append(c1)

    c1 = BiochemicalReaction(
        uid="conversion_Reaction4",
        xref="http://identifiers.org/obo.go/GO:0006412",
        participant_stoichiometry="RIGHT_0_conversion_Reaction4_PX_STOICHIOMETRY",
        display_name="translation of LacI",
        right="RIGHT_0_conversion_Reaction4_PX",
    )
    objects.append(c1)

    c1 = BiochemicalReaction(
        uid="conversion_Reaction5",
        xref="http://identifiers.org/obo.go/GO:0006412",
        participant_stoichiometry="RIGHT_0_conversion_Reaction5_PY_STOICHIOMETRY",
        display_name="translation of TetR",
        right="RIGHT_0_conversion_Reaction5_PY",
    )
    objects.append(c1)

    p1 = PhysicalEntity(
        uid="LEFT_0_conversion_Reaction2_Y",
        cellular_location="cell",
        display_name="Y",
        member_physical_entity="Y",
    )
    objects.append(p1)

    c1 = BiochemicalReaction(
        uid="conversion_Reaction6",
        xref="http://identifiers.org/obo.go/GO:0006412",
        participant_stoichiometry="RIGHT_0_conversion_Reaction6_PZ_STOICHIOMETRY",
        display_name="translation of CI",
        right="RIGHT_0_conversion_Reaction6_PZ",
    )
    objects.append(c1)

    c1 = BiochemicalReaction(
        uid="conversion_Reaction7",
        xref="http://identifiers.org/obo.go/GO:0030163",
        participant_stoichiometry="LEFT_0_conversion_Reaction7_PX_STOICHIOMETRY",
        display_name="degradation of LacI",
        left="LEFT_0_conversion_Reaction7_PX",
    )
    objects.append(c1)

    s1 = Stoichiometry(
        uid="LEFT_0_conversion_Reaction9_PZ_STOICHIOMETRY",
        physical_entity="LEFT_0_conversion_Reaction9_PZ",
        stoichiometric_coefficient="1",
    )
    objects.append(s1)

    c1 = BiochemicalReaction(
        uid="conversion_Reaction8",
        xref="http://identifiers.org/obo.go/GO:0030163",
        participant_stoichiometry="LEFT_0_conversion_Reaction8_PY_STOICHIOMETRY",
        display_name="degradation of TetR",
        left="LEFT_0_conversion_Reaction8_PY",
    )
    objects.append(c1)

    c1 = BiochemicalReaction(
        uid="conversion_Reaction9",
        xref="http://identifiers.org/obo.go/GO:0030163",
        participant_stoichiometry="LEFT_0_conversion_Reaction9_PZ_STOICHIOMETRY",
        display_name="degradation of CI",
        left="LEFT_0_conversion_Reaction9_PZ",
    )
    objects.append(c1)

    u1 = UnificationXref(
        uid="http://identifiers.org/taxonomy/562", id="562", db="Taxonomy"
    )
    objects.append(u1)

    s1 = Stoichiometry(
        uid="LEFT_0_conversion_Reaction1_X_STOICHIOMETRY",
        physical_entity="LEFT_0_conversion_Reaction1_X",
        stoichiometric_coefficient="1",
    )
    objects.append(s1)

    p1 = PhysicalEntity(
        uid="RIGHT_0_conversion_Reaction4_PX",
        cellular_location="cell",
        display_name="PX",
        member_physical_entity="PX",
    )
    objects.append(p1)

    u1 = UnificationXref(
        uid="http://identifiers.org/uniprot/P03034",
        id="P03034",
        db="UniProt Knowledgebase",
    )
    objects.append(u1)

    p1 = PhysicalEntity(
        uid="PEP0_control_Reaction10PZ",
        cellular_location="cell",
        display_name="PZ",
        member_physical_entity="PZ",
    )
    objects.append(p1)

    s1 = Stoichiometry(
        uid="LEFT_0_conversion_Reaction3_Z_STOICHIOMETRY",
        physical_entity="LEFT_0_conversion_Reaction3_Z",
        stoichiometric_coefficient="1",
    )
    objects.append(s1)

    p1 = PhysicalEntity(
        uid="RIGHT_0_conversion_Reaction10_X",
        cellular_location="cell",
        display_name="X",
        member_physical_entity="X",
    )
    objects.append(p1)

    b1 = BioSource(uid="biosource_1", xref="http://identifiers.org/taxonomy/562")
    objects.append(b1)

    p1 = PhysicalEntity(
        uid="RIGHT_0_conversion_Reaction11_Y",
        cellular_location="cell",
        display_name="Y",
        member_physical_entity="Y",
    )
    objects.append(p1)

    p1 = PhysicalEntity(
        uid="RIGHT_0_conversion_Reaction12_Z",
        cellular_location="cell",
        display_name="Z",
        member_physical_entity="Z",
    )
    objects.append(p1)

    s1 = Stoichiometry(
        uid="LEFT_0_conversion_Reaction8_PY_STOICHIOMETRY",
        physical_entity="LEFT_0_conversion_Reaction8_PY",
        stoichiometric_coefficient="1",
    )
    objects.append(s1)

    p1 = PhysicalEntity(
        uid="RIGHT_0_conversion_Reaction6_PZ",
        cellular_location="cell",
        display_name="PZ",
        member_physical_entity="PZ",
    )
    objects.append(p1)

    p1 = PhysicalEntity(
        uid="PEP0_control_Reaction6Z",
        cellular_location="cell",
        display_name="Z",
        member_physical_entity="Z",
    )
    objects.append(p1)

    p1 = PhysicalEntity(
        uid="PEP0_control_Reaction11PX",
        cellular_location="cell",
        display_name="PX",
        member_physical_entity="PX",
    )
    objects.append(p1)

    r1 = Rna(
        uid="Y",
        xref=[
            "http://identifiers.org/chebi/CHEBI:33699",
            "http://identifiers.org/kegg.compound/C00046",
            "http://identifiers.org/uniprot/P04483",
        ],
        display_name="TetR mRNA",
    )
    objects.append(r1)

    r1 = Rna(
        uid="X",
        xref=[
            "http://identifiers.org/chebi/CHEBI:33699",
            "http://identifiers.org/kegg.compound/C00046",
            "http://identifiers.org/uniprot/P03023",
        ],
        display_name="LacI mRNA",
    )
    objects.append(r1)

    s1 = Stoichiometry(
        uid="RIGHT_0_conversion_Reaction5_PY_STOICHIOMETRY",
        physical_entity="RIGHT_0_conversion_Reaction5_PY",
        stoichiometric_coefficient="1",
    )
    objects.append(s1)

    r1 = Rna(
        uid="Z",
        xref=[
            "http://identifiers.org/chebi/CHEBI:33699",
            "http://identifiers.org/kegg.compound/C00046",
            "http://identifiers.org/uniprot/P03034",
        ],
        display_name="cI mRNA",
    )
    objects.append(r1)

    c1 = Control(
        uid="control_Reaction6_Z_0",
        controlled="conversion_Reaction6",
        control_type="INHIBITION",
        controller="PEP0_control_Reaction6Z",
    )
    objects.append(c1)

    c1 = Control(
        uid="control_Reaction12_PY_0",
        controlled="conversion_Reaction12",
        control_type="INHIBITION",
        controller="PEP0_control_Reaction12PY",
    )
    objects.append(c1)

    p1 = PhysicalEntity(
        uid="PEP0_control_Reaction12PY",
        cellular_location="cell",
        display_name="PY",
        member_physical_entity="PY",
    )
    objects.append(p1)

    u1 = UnificationXref(
        uid="http://identifiers.org/biomodels.db/MODEL6615351360",
        id="MODEL6615351360",
        db="BioModels Database",
    )
    objects.append(u1)

    u1 = UnificationXref(
        uid="http://identifiers.org/uniprot/P03023",
        id="P03023",
        db="UniProt Knowledgebase",
    )
    objects.append(u1)

    c1 = Control(
        uid="control_Reaction10_PZ_0",
        controlled="conversion_Reaction10",
        control_type="INHIBITION",
        controller="PEP0_control_Reaction10PZ",
    )
    objects.append(c1)

    u1 = UnificationXref(
        uid="http://identifiers.org/obo.go/GO:0006412",
        id="GO:0006412",
        db="Gene Ontology",
    )
    objects.append(u1)

    u1 = UnificationXref(
        uid="http://identifiers.org/obo.go/GO:0040029",
        id="GO:0040029",
        db="Gene Ontology",
    )
    objects.append(u1)

    u1 = UnificationXref(
        uid="http://identifiers.org/obo.go/GO:0006351",
        id="GO:0006351",
        db="Gene Ontology",
    )
    objects.append(u1)

    c1 = Control(
        uid="control_Reaction5_Y_0",
        controlled="conversion_Reaction5",
        control_type="INHIBITION",
        controller="PEP0_control_Reaction5Y",
    )
    objects.append(c1)

    p1 = PhysicalEntity(
        uid="LEFT_0_conversion_Reaction1_X",
        cellular_location="cell",
        display_name="X",
        member_physical_entity="X",
    )
    objects.append(p1)

    p1 = PhysicalEntity(
        uid="PEP0_control_Reaction5Y",
        cellular_location="cell",
        display_name="Y",
        member_physical_entity="Y",
    )
    objects.append(p1)

    c1 = Control(
        uid="control_Reaction11_PX_0",
        controlled="conversion_Reaction11",
        control_type="INHIBITION",
        controller="PEP0_control_Reaction11PX",
    )
    objects.append(c1)

    c1 = BiochemicalReaction(
        uid="conversion_Reaction1",
        xref="http://identifiers.org/obo.go/GO:0006402",
        participant_stoichiometry="LEFT_0_conversion_Reaction1_X_STOICHIOMETRY",
        display_name="degradation of LacI transcripts",
        left="LEFT_0_conversion_Reaction1_X",
    )
    objects.append(c1)

    u1 = UnificationXref(
        uid="http://identifiers.org/uniprot/P04483",
        id="P04483",
        db="UniProt Knowledgebase",
    )
    objects.append(u1)
    # '''
    # <bp:BiochemicalReaction rdf:about="conversion_Reaction12">
    #  <bp:xref rdf:resource="http://identifiers.org/obo.go/GO:0006351" />
    #  <bp:participantStoichiometry rdf:resource="RIGHT_0_conversion_Reaction12_Z_STOICHIOMETRY" />
    #  <bp:displayName rdf:datatype = "http://www.w3.org/2001/XMLSchema#string">transcription of CI</bp:displayName>
    #  <bp:right rdf:resource="RIGHT_0_conversion_Reaction12_Z" />
    # </bp:BiochemicalReaction>
    # '''
    # b1=BiochemicalReaction(uid="conversion_Reaction12", displayName="transcription of CI")
    # objects.append(b1)

    model = BioPaxModel(
        objects={o.uid: o for o in objects},
        xml_base="http://www.biopax.org/release/biopax-level3.owl#",
    )
    owl_str: str = model_to_owl_str(model)

    with open(biopax_path, "w") as f_biopax:
        f_biopax.write(owl_str)

    print("-" * 80)
    print(owl_str)
    print("-" * 80)

    return model


# <codecell>
if __name__ == "__main__":
    from combine_notebooks import RESULTS_DIR

    create_repressilator(biopax_path=RESULTS_DIR / "repressilator_biopax.owl")



"""Example script for creating SBML repressilator with neuroml.

https://neuroml.org/
https://docs.neuroml.org/Userdocs/CreatingNeuroMLModels.html

https://pyneuroml.readthedocs.io/en/latest/pyneuroml.html#module-pyneuroml.pynml
https://docs.biosimulators.org/Biosimulators_pyNeuroML/installation.html
https://github.com/NeuroML/Documentation/blob/master/source/Userdocs/NML2_examples/SingleNeuron.ipynb
"""



# <markdowncell>
# Example script for creating SBML repressilator with sbmlutils.

# https://github.com/matthiaskoenig/libsbgn-python
# <codecell>
from pathlib import Path

from sbmlutils.factory import (
    AssignmentRule,
    Compartment,
    FactoryResult,
    Model,
    ModelUnits,
    Parameter,
    Reaction,
    Species,
    Units,
    create_model,
)
from sbmlutils.metadata import *
from sbmlutils.metadata import BQB

from combine_notebooks import RESULTS_DIR


# <codecell>
model = Model(
    "repressilator_sbmlutils",
    units=Units,
    model_units=ModelUnits(
        time=Units.second,
        substance=Units.mole,
        extent=Units.mole,
        volume=Units.litre,
    ),
    compartments=[
        Compartment(
            sid="cell",
            value=1.0,
            sboTerm="SBO:0000290",
            annotations=[(BQB.IS, "obo.go/GO:0005623")],
        )
    ],
    # obo.go/GO:3A0005623
    species=[
        Species(
            sid="PX",
            compartment="cell",
            name="LacI protein",
            sboTerm="SBO:0000252",
            initialAmount=0,
            hasOnlySubstanceUnits=True,
            annotations=[(BQB.IS, "uniprot/P03023")],
        ),
        Species(
            sid="PY",
            compartment="cell",
            name="TetR protein",
            sboTerm="SBO:0000252",
            initialAmount=0,
            hasOnlySubstanceUnits=True,
            annotations=[(BQB.IS, "uniprot/P04483")],
        ),
        Species(
            sid="PZ",
            compartment="cell",
            name="cI protein",
            sboTerm="SBO:0000252",
            initialAmount=0,
            hasOnlySubstanceUnits=True,
            annotations=[(BQB.IS, "uniprot/P03034")],
        ),
        Species(
            sid="X",
            compartment="cell",
            name="Lacl mRNA",
            sboTerm="SBO:0000250",
            initialAmount=20,
            hasOnlySubstanceUnits=True,
            annotations=[
                (BQB.IS_VERSION_OF, "chebi/CHEBI:33699"),
                (BQB.IS_VERSION_OF, "kegg.compound/C00046"),
                (BQB.ENCODES, "uniprot/P03023"),
            ],
        ),
        Species(
            sid="Y",
            compartment="cell",
            name="TetR mRNA",
            sboTerm="SBO:0000250",
            initialAmount=20,
            hasOnlySubstanceUnits=True,
            annotations=[
                (BQB.IS_VERSION_OF, "chebi/CHEBI:33699"),
                (BQB.IS_VERSION_OF, "kegg.compound/C00046"),
                (BQB.ENCODES, "uniprot/P04483"),
            ],
        ),
        Species(
            sid="Z",
            compartment="cell",
            name="cl mRNA",
            sboTerm="SBO:0000250",
            initialAmount=0,
            hasOnlySubstanceUnits=True,
            annotations=[
                (BQB.IS_VERSION_OF, "chebi/CHEBI:33699"),
                (BQB.IS_VERSION_OF, "kegg.compound/C00046"),
                (BQB.ENCODES, "uniprot/P03034"),
            ],
        ),
    ],
    parameters=[
        Parameter(sid="tau_mRNA", name="mRNA half life", constant=True, value=2.0),
        Parameter(sid="kd_mRNA", name="kd_mRNA", constant=False, sboTerm="SBO:0000356"),
        Parameter(sid="k_tl", name="k_tl", constant=False, sboTerm="SBO:0000016"),
        Parameter(
            sid="t_ave",
            name="average mRNA life time",
            constant=False,
            sboTerm="SBO:0000348",
        ),
        Parameter(sid="eff", name="translation efficiency", constant=True, value=20),
        Parameter(sid="kd_prot", name="kd_prot", constant=False, sboTerm="SBO:0000356"),
        Parameter(sid="tau_prot", name="protien half life", constant=True, value=10),
        Parameter(sid="a0_tr", name="a0_tr", constant=False, sboTerm="SBO:0000485"),
        Parameter(sid="ps_0", name="tps_repr", constant=True, value=0.0005),
        Parameter(sid="ps_a", name="tps_active", constant=True, value=0.5),
        Parameter(sid="KM", name="KM", constant=True, value=40),
        Parameter(sid="n", name="n", constant=True, value=2.0),
        Parameter(sid="a_tr", name="a_tr", constant=False, sboTerm="SBO:0000186"),
    ],
    rules=[
        AssignmentRule(
            sid="kd_mRNA", value="ln(2) / tau_mRNA", name="kd_MRNA", sboTerm=None
        ),
        AssignmentRule(
            sid="t_ave", value="tau_mRNA / ln(2)", name="t_ave", sboTerm=None
        ),
        AssignmentRule(sid="k_tl", value="eff / t_ave", name="k_tl", sboTerm=None),
        AssignmentRule(
            sid="kd_prot", value="ln(2) / tau_prot", name="kd_prot", sboTerm=None
        ),
        AssignmentRule(sid="a0_tr", value="ps_0 * 60", name="a0_tr", sboTerm=None),
        AssignmentRule(
            sid="a_tr", value="(ps_a - ps_0) * 60", name="a_tr", sboTerm=None
        ),
    ],
    reactions=[
        Reaction(
            sid="Reaction1",
            sboTerm="SBO:0000179",
            name="degradation of LacI transcripts",
            reversible=False,
            equation="X -> ",
            formula=("kd_mRNA * X", None),
            annotations=[(BQB.IS_VERSION_OF, "obo.go/GO:0006402")],
        ),
        Reaction(
            sid="Reaction2",
            sboTerm="SBO:0000179",
            name="degradation of TetR transcripts",
            reversible=False,
            equation="Y -> ",
            formula=("kd_mRNA * Y", None),
            annotations=[(BQB.IS_VERSION_OF, "obo.go/GO:0006402")],
        ),
        Reaction(
            sid="Reaction3",
            sboTerm="SBO:0000179",
            name="degradation of CI transcripts",
            reversible=False,
            equation="Z -> ",
            formula=("kd_mRNA * Z", None),
            annotations=[(BQB.IS_VERSION_OF, "obo.go/GO:0006402")],
        ),
        Reaction(
            sid="Reaction4",
            sboTerm="SBO:0000184",
            name="translation of LacI",
            reversible=False,
            equation=" -> PX [X]",
            formula=("k_tl * X", None),
            annotations=[(BQB.IS_VERSION_OF, "obo.go/GO:0006412")],
        ),
        Reaction(
            sid="Reaction5",
            sboTerm="SBO:0000184",
            name="translation of TetR",
            reversible=False,
            equation=" -> PY [Y]",
            formula=("k_tl * Y", None),
            annotations=[(BQB.IS_VERSION_OF, "obo.go/GO:0006412")],
        ),
        Reaction(
            sid="Reaction6",
            sboTerm="SBO:0000184",
            name="translation of CI",
            reversible=False,
            equation=" -> PZ [Z]",
            formula=("k_tl *Z", None),
            annotations=[(BQB.IS_VERSION_OF, "obo.go/GO:0006412")],
        ),
        Reaction(
            sid="Reaction7",
            sboTerm="SBO:0000179",
            name="degradation of LacI",
            reversible=False,
            equation="PX -> ",
            formula=("kd_prot * PX", None),
            annotations=[(BQB.IS_VERSION_OF, "obo.go/GO:0030163")],
        ),
        Reaction(
            sid="Reaction8",
            sboTerm="SBO:0000179",
            name="degradation of TetR",
            reversible=False,
            equation="PY -> ",
            formula=("kd_prot * PY", None),
            annotations=[(BQB.IS_VERSION_OF, "obo.go/GO:0030163")],
        ),
        Reaction(
            sid="Reaction9",
            sboTerm="SBO:0000179",
            name="degradation of CI",
            reversible=False,
            equation="PZ -> ",
            formula=("kd_prot * PZ", None),
            annotations=[(BQB.IS_VERSION_OF, "obo.go/GO:0030163")],
        ),
        Reaction(
            sid="Reaction10",
            sboTerm="SBO:0000183",
            name="transcription of LacI",
            reversible=False,
            equation=" -> X [PZ]",
            formula=(
                "a0_tr + (a_tr * power(KM, n)) / (power(KM, n) + power(PZ, n))",
                None,
            ),
            annotations=[(BQB.IS_VERSION_OF, "obo.go/GO:0006351")],
        ),
        Reaction(
            sid="Reaction11",
            sboTerm="SBO:0000183",
            name="transcription of TetR",
            reversible=False,
            equation=" -> Y [PX]",
            formula=(
                "a0_tr + (a_tr * power(KM, n)) / (power(KM, n) + power(PX, n))",
                None,
            ),
            annotations=[(BQB.IS_VERSION_OF, "obo.go/GO:0006351")],
        ),
        Reaction(
            sid="Reaction12",
            sboTerm="SBO:0000184",
            name="transcription of CI",
            reversible=False,
            equation=" -> Z [PY]",
            formula=(
                "a0_tr + (a_tr * power(KM, n)) / (power(KM, n) + power(PY, n))",
                None,
            ),
            annotations=[(BQB.IS_VERSION_OF, "obo.go/GO:0006351")],
        ),
    ],
)

# <codecell>


def create_repressilator(sbml_path: Path) -> FactoryResult:
    """Create the repressilator model in the results_dir."""
    results = create_model(
        models=model,
        output_dir=sbml_path.parent,
        filename=sbml_path.name,
        tmp=False,
        units_consistency=False,
        sbml_level=2,
        sbml_version=3,
    )
    return results


# <codecell>
if __name__ == "__main__":
    # RESOURCES_DIR: Path = Path(__file__).parent / "resources"
    # RESULTS_DIR: Path = RESOURCES_DIR / "results"

    create_repressilator(sbml_path=RESULTS_DIR / "repressilator_sbmlutils.xml")



# <markdowncell>
# Create repressilator using SED-ML

# SED-ML encodes in a computer-readable exchange format the information required by MIASE to enable reproduction of simulation experiments.

# https://sed-ml.org/
# <codecell>
from pathlib import Path

import libsedml


# <codecell>


def create_repressilator(sedml_path: Path) -> libsedml.SedDocument:
    """Create repressilator using SED-ML."""
    # create the document
    doc = libsedml.SedDocument(1, 4)

    # create a first model referencing an sbml file

    """
    <listOfSimulations>
        <uniformTimeCourse id="sim1" initialTime="0" outputStartTime="0" outputEndTime="1000" numberOfPoints="1000">
          <algorithm kisaoID="KISAO:0000019"/>
        </uniformTimeCourse>
      </listOfSimulations>
    """
    # create simulation
    tc = doc.createUniformTimeCourse()
    tc.setId("sim1")
    tc.setInitialTime(0.0)
    tc.setOutputStartTime(0.0)
    tc.setOutputEndTime(1000.0)
    tc.setNumberOfPoints(1000)
    # need to set the correct KISAO Term
    alg = tc.createAlgorithm()
    alg.setKisaoID("KISAO:0000019")

    """
    <listOfModels>
    <model id="model1" language="urn:sedml:language:sbml.level-3.version-1" source="https://www.ebi.ac.uk/biomodels/model/download/BIOMD0000000012?filename=BIOMD0000000012_url.xml"/>
    <model id="model2" language="urn:sedml:language:sbml.level-3.version-1" source="#model1">
      <listOfChanges>
        <changeAttribute target="/sbml:sbml/sbml:model/sbml:listOfParameters/sbml:parameter[@id='ps_0']/@value" newValue="1.3e-05"/>
        <changeAttribute target="/sbml:sbml/sbml:model/sbml:listOfParameters/sbml:parameter[@id='ps_a']/@value" newValue="0.013"/>
      </listOfChanges>
    </model>
    </listOfModels>
    """
    # create a first model referencing an sbml file
    model = doc.createModel()
    model.setId("model1")
    model.setSource(
        "https://www.ebi.ac.uk/biomodels/model/download/BIOMD0000000012?filename=BIOMD0000000012_url.xml"
    )
    model.setLanguage("urn:sedml:language:sbml.level-3.version-1")

    # create a second model modifying a variable of that other sbml file
    model = doc.createModel()
    model.setId("model2")
    model.setSource("#model1")
    model.setLanguage("urn:sedml:language:sbml.level-3.version-1")

    change = model.createChangeAttribute()
    change.setTarget(
        "/sbml:sbml/sbml:model/sbml:listOfParameters/sbml:parameter[@id='ps_0']/@value"
    )
    change.setNewValue("1.3e-05")

    change = model.createChangeAttribute()
    change.setTarget(
        "/sbml:sbml/sbml:model/sbml:listOfParameters/sbml:parameter[@id='ps_a']/@value"
    )
    change.setNewValue("0.013")

    """
    <listOfTasks>
    <task id="task1" modelReference="model1" simulationReference="sim1"/>
    <task id="task2" modelReference="model2" simulationReference="sim1"/>
    </listOfTasks>
    """
    # create a task that uses the simulation and the model above
    task = doc.createTask()
    task.setId("task1")
    task.setModelReference("model1")
    task.setSimulationReference("sim1")

    # create a task that uses the simulation and the model above
    task = doc.createTask()
    task.setId("task2")
    task.setModelReference("model2")
    task.setSimulationReference("sim1")

    """
    <dataGenerator id="dg_0_0_0" name="task1.time">
      <listOfVariables>
        <variable id="task1_____time" symbol="urn:sedml:symbol:time" taskReference="task1"/>
      </listOfVariables>
      <math xmlns="http://www.w3.org/1998/Math/MathML">
        <ci> task1_____time </ci>
      </math>
    </dataGenerator>
    """
    # add a DataGenerator to hold the output for time
    dg = doc.createDataGenerator()
    dg.setId("dg_0_0_0")
    dg.setName("task1.time")
    var = dg.createVariable()
    var.setId("task1_____time")
    var.setTaskReference("task1")
    var.setSymbol("urn:sedml:symbol:time")
    dg.setMath(libsedml.parseFormula("task1_____time"))

    dg = doc.createDataGenerator()
    dg.setId("dg_0_0_1")
    dg.setName("PX (lacI)")
    var = dg.createVariable()
    var.setId("task1_____PX")
    var.setTarget("/sbml:sbml/sbml:model/sbml:listOfSpecies/sbml:species[@id='PX']")
    var.setTaskReference("task1")
    var.setModelReference("model1")
    dg.setMath(libsedml.parseFormula("task1_____PX"))

    dg = doc.createDataGenerator()
    dg.setId("dg_0_1_1")
    dg.setName("PZ (cI)")
    var = dg.createVariable()
    var.setId("task1_____PZ")
    var.setTarget("/sbml:sbml/sbml:model/sbml:listOfSpecies/sbml:species[@id='PZ']")
    var.setTaskReference("task1")
    var.setModelReference("model1")
    dg.setMath(libsedml.parseFormula("task1_____PZ"))

    dg = doc.createDataGenerator()
    dg.setId("dg_0_2_1")
    dg.setName("PY (tetR)")
    var = dg.createVariable()
    var.setId("task1_____PY")
    var.setTarget("/sbml:sbml/sbml:model/sbml:listOfSpecies/sbml:species[@id='PY']")
    var.setTaskReference("task1")
    var.setModelReference("model1")
    dg.setMath(libsedml.parseFormula("task1_____PY"))

    dg = doc.createDataGenerator()
    dg.setId("dg_1_0_0")
    dg.setName("time")
    var = dg.createVariable()
    var.setId("task2_____time")
    var.setSymbol("urn:sedml:symbol:time")
    var.setTaskReference("task2")
    dg.setMath(libsedml.parseFormula("task2_____time"))

    dg = doc.createDataGenerator()
    dg.setId("dg_1_0_1")
    dg.setName("PX (lacI)")
    var = dg.createVariable()
    var.setId("task2_____PX")
    var.setTarget("/sbml:sbml/sbml:model/sbml:listOfSpecies/sbml:species[@id='PX']")
    var.setTaskReference("task2")
    var.setModelReference("model2")
    dg.setMath(libsedml.parseFormula("task2_____PX"))

    dg = doc.createDataGenerator()
    dg.setId("dg_1_1_1")
    dg.setName("PZ (cI)")
    var = dg.createVariable()
    var.setId("task2_____PZ")
    var.setTarget("/sbml:sbml/sbml:model/sbml:listOfSpecies/sbml:species[@id='PZ']")
    var.setTaskReference("task2")
    var.setModelReference("model2")
    dg.setMath(libsedml.parseFormula("task2_____PZ"))

    dg = doc.createDataGenerator()
    dg.setId("dg_1_2_1")
    dg.setName("PY (tetR)")
    var = dg.createVariable()
    var.setId("task2_____PY")
    var.setTarget("/sbml:sbml/sbml:model/sbml:listOfSpecies/sbml:species[@id='PY']")
    var.setTaskReference("task2")
    var.setModelReference("model2")
    dg.setMath(libsedml.parseFormula("task2_____PY"))

    dg = doc.createDataGenerator()
    dg.setId("dg_2_0_0")
    dg.setName("PX/max(PX) (lacI normalized)")
    var = dg.createVariable()
    var.setId("task1_____PX")
    var.setTarget("/sbml:sbml/sbml:model/sbml:listOfSpecies/sbml:species[@id='PX']")
    var.setTaskReference("task1")
    var.setModelReference("model1")
    var = dg.createVariable()
    var.setId("task1_____PX_max")
    var.setTarget("/sbml:sbml/sbml:model/sbml:listOfSpecies/sbml:species[@id='PX']")
    var.setTaskReference("task1")
    var.setModelReference("model1")
    var.setDimensionTerm("KISAO:0000828")
    dg.setMath(libsedml.parseFormula("task1_____PX / task1_____PX_max"))

    dg = doc.createDataGenerator()
    dg.setId("dg_2_0_1")
    dg.setName("PZ/max(PZ) (cI normalized)")
    var = dg.createVariable()
    var.setId("task1_____PZ")
    var.setTarget("/sbml:sbml/sbml:model/sbml:listOfSpecies/sbml:species[@id='PZ']")
    var.setTaskReference("task1")
    var.setModelReference("model1")
    var = dg.createVariable()
    var.setId("task1_____PZ_max")
    var.setTarget("/sbml:sbml/sbml:model/sbml:listOfSpecies/sbml:species[@id='PX']")
    var.setTaskReference("task1")
    var.setModelReference("model1")
    var.setDimensionTerm("KISAO:0000828")
    dg.setMath(libsedml.parseFormula("task1_____PZ / task1_____PZ_max"))

    dg = doc.createDataGenerator()
    dg.setId("dg_2_1_0")
    dg.setName("PY/max(PY) (tetR normalized)")
    var = dg.createVariable()
    var.setId("task1_____PY")
    var.setTarget("/sbml:sbml/sbml:model/sbml:listOfSpecies/sbml:species[@id='PY']")
    var.setTaskReference("task1")
    var.setModelReference("model1")
    var = dg.createVariable()
    var.setId("task1_____PY_max")
    var.setTarget("/sbml:sbml/sbml:model/sbml:listOfSpecies/sbml:species[@id='PY']")
    var.setTaskReference("task1")
    var.setModelReference("model1")
    var.setDimensionTerm("KISAO:0000828")
    dg.setMath(libsedml.parseFormula("task1_____PY / task1_____PY_max"))

    # add a 2d plot
    plot = doc.createPlot2D()
    plot.setId("timecourse")
    plot.setName("Timecourse of repressilator")
    curve = plot.createCurve()
    curve.setId("plot_0__plot_0_0_0__plot_0_0_1")
    curve.setLogX(False)
    curve.setLogY(False)
    curve.setXDataReference("dg_0_0_0")
    curve.setYDataReference("dg_0_0_1")
    curve = plot.createCurve()
    curve.setId("plot_0__plot_0_0_0__plot_0_1_1")
    curve.setLogX(False)
    curve.setLogY(False)
    curve.setXDataReference("dg_0_0_0")
    curve.setYDataReference("dg_0_1_1")
    curve = plot.createCurve()
    curve.setId("plot_0__plot_0_0_0__plot_0_2_1")
    curve.setLogX(False)
    curve.setLogY(False)
    curve.setXDataReference("dg_0_0_0")
    curve.setYDataReference("dg_0_2_1")

    # add a 2d plot
    plot = doc.createPlot2D()
    plot.setId("preprocessing")
    plot.setName("Timecourse after pre-processing")
    curve = plot.createCurve()
    curve.setId("plot_1__plot_1_0_0__plot_1_0_1")
    curve.setLogX(False)
    curve.setLogY(False)
    curve.setXDataReference("dg_1_0_0")
    curve.setYDataReference("dg_1_0_1")
    curve = plot.createCurve()
    curve.setId("plot_1__plot_1_0_0__plot_1_1_1")
    curve.setLogX(False)
    curve.setLogY(False)
    curve.setXDataReference("dg_1_0_0")
    curve.setYDataReference("dg_1_1_1")
    curve = plot.createCurve()
    curve.setId("plot_1__plot_1_0_0__plot_1_2_1")
    curve.setLogX(False)
    curve.setLogY(False)
    curve.setXDataReference("dg_1_0_0")
    curve.setYDataReference("dg_1_2_1")

    # add a 2d plot
    plot = doc.createPlot2D()
    plot.setId("postprocessing")
    plot.setName("Timecourse after post-processing")
    curve = plot.createCurve()
    curve.setId("plot_2__plot_2_0_0__plot_2_0_1")
    curve.setLogX(False)
    curve.setLogY(False)
    curve.setXDataReference("dg_2_0_0")
    curve.setYDataReference("dg_2_0_1")
    curve = plot.createCurve()
    curve.setId("plot_2__plot_2_1_0__plot_2_0_0")
    curve.setLogX(False)
    curve.setLogY(False)
    curve.setXDataReference("dg_2_1_0")
    curve.setYDataReference("dg_2_0_0")
    curve = plot.createCurve()
    curve.setId("plot_2__plot_2_0_1__plot_2_1_0")
    curve.setLogX(False)
    curve.setLogY(False)
    curve.setXDataReference("dg_2_0_1")
    curve.setYDataReference("dg_2_1_0")

    # write doc
    libsedml.writeSedML(doc, str(sedml_path))
    return doc


# <codecell>
if __name__ == "__main__":
    from combine_notebooks import RESULTS_DIR

    create_repressilator(sedml_path=RESULTS_DIR / "repressilator_sedml.xml")



"""Validation function."""
from typing import List

import libsedml

from combine_notebooks import RESULTS_DIR


def validate_sedml(doc: libsedml.SedDocument) -> None:
    """Validate SED-ML."""
    # set the unit checking, similar for the other settings

    n_errors: int = doc.getNumErrors()
    errors: List[libsedml.SedError] = list()
    warnings: List[libsedml.SedError] = list()
    for k in range(n_errors):
        error: libsedml.SedError = doc.getError(k)
        severity = error.getSeverity()
        if (severity == libsedml.LIBSEDML_SEV_ERROR) or (
            severity == libsedml.LIBSEDML_SEV_FATAL
        ):
            errors.append(error)
        else:
            warnings.append(error)
    # print results
    print("-" * 80)
    print(f"{'validation error(s)':<25}: {len(errors)}")
    print(f"{'validation warning(s)':<25}: {len(warnings)}")
    if len(errors) > 0:
        print("--- errors ---")
        for kerr in enumerate(errors):
            print(f"E{kerr}: {error.getCategoryAsString()} L{error.getLine()}")
            print(
                f"[{error.getSeverityAsString()}] {error.getShortMessage()} | {error.getMessage()}"
            )
    if len(warnings) > 0:
        print("--- warnings ---")
        for kwarn in enumerate(warnings):
            print(f"E{kwarn}: {error.getCategoryAsString()} L{error.getLine()}")
            print(
                f"[{error.getSeverityAsString()}] {error.getShortMessage()} | {error.getMessage()}"
            )
    print("-" * 80)


if __name__ == "__main__":

    doc: libsedml.SedDocument = libsedml.readSedMLFromFile(
        str(RESULTS_DIR / "repressilator_sedml.xml")
    )
    validate_sedml(doc)
